@@ -2,7 +2,7 @@
 
 cd ..
 mkdir build
-CSC=/usr/bin/csc GACUTIL=/usr/bin/gacutil CXX=clang++ CC=clang python scripts/mk_make.py  --java --python
+CXX=clang++ CC=clang python scripts/mk_make.py  --java --python
 cd build
 make
 make test-z3
