@@ -158,7 +158,10 @@ extern "C" {
         model_ref new_m;
         Z3_model_ref * m_ref = alloc(Z3_model_ref, *mk_c(c)); 
         mk_c(c)->save_object(m_ref);
-        if (m) m_ref->m_model = to_model_ref(m)->copy(); 
+        if (m)
+            m_ref->m_model = to_model_ref(m)->copy();
+        else
+            m_ref->m_model = alloc(model, mk_c(c)->m());
         if (to_goal_ref(g)->mc()) 
             (*to_goal_ref(g)->mc())(m_ref->m_model);
         RETURN_Z3(of_model(m_ref));
@@ -185,7 +188,7 @@ extern "C" {
         std::ostringstream buffer;
         to_goal_ref(g)->display(buffer);
         // Hack for removing the trailing '\n'
-        std::string result = buffer.str();
+        std::string result = std::move(buffer).str();
         SASSERT(result.size() > 0);
         result.resize(result.size()-1);
         return mk_c(c)->mk_external_string(std::move(result));
@@ -203,7 +206,7 @@ extern "C" {
         }
         to_goal_ref(g)->display_dimacs(buffer, include_names);
         // Hack for removing the trailing '\n'
-        std::string result = buffer.str();
+        std::string result = std::move(buffer).str();
         SASSERT(result.size() > 0);
         result.resize(result.size()-1);
         return mk_c(c)->mk_external_string(std::move(result));
