@@ -154,7 +154,7 @@ extern "C" {
         bool     use_ctrl_c  = to_optimize_ptr(o)->get_params().get_bool("ctrl_c", true);
         api::context::set_interruptable si(*(mk_c(c)), eh);        
         {
-            scoped_ctrl_c ctrlc(eh, false, use_ctrl_c);
+            scoped_ctrl_c ctrlc(eh, use_ctrl_c);
             scoped_timer timer(timeout, &eh);
             scoped_rlimit _rlimit(mk_c(c)->m().limit(), rlimit);
             try {
@@ -168,7 +168,7 @@ extern "C" {
                 }
                 r = l_undef;
                 if (!mk_c(c)->m().inc()) {
-                    to_optimize_ptr(o)->set_reason_unknown(ex.msg());
+                    to_optimize_ptr(o)->set_reason_unknown(ex.what());
                 }
                 else {
                     mk_c(c)->handle_exception(ex);
@@ -313,7 +313,7 @@ extern "C" {
         param_descrs descrs;
         to_optimize_ptr(d)->collect_param_descrs(descrs);
         descrs.display(buffer);
-        return mk_c(c)->mk_external_string(buffer.str());
+        return mk_c(c)->mk_external_string(std::move(buffer).str());
         Z3_CATCH_RETURN("");
     }
 
@@ -364,15 +364,20 @@ extern "C" {
             }        
         }
         catch (z3_exception& e) {
-            errstrm << e.msg();
+            errstrm << e.what();
             ctx = nullptr;
             SET_ERROR_CODE(Z3_PARSER_ERROR, errstrm.str());
             return;
         }
 
-        for (expr * e : ctx->assertions()) {
-            to_optimize_ptr(opt)->add_hard_constraint(e);
-        }
+        auto o = to_optimize_ptr(opt);
+
+        for (auto const& [asr, an] : ctx->tracked_assertions())
+            if (an)
+                o->add_hard_constraint(asr, an);
+            else
+                o->add_hard_constraint(asr);
+
     }
 
 
