@@ -128,6 +128,7 @@ extern "C" {
 
     static Z3_ast_vector Z3_parser_context_parse_stream(Z3_context c, scoped_ptr<cmd_context>& ctx, bool owned, std::istream& is) {
         Z3_TRY;
+        RESET_ERROR_CODE();
         ast_manager& m = mk_c(c)->m();
         Z3_ast_vector_ref * v = alloc(Z3_ast_vector_ref, *mk_c(c), m);        
         mk_c(c)->save_object(v);        
@@ -144,7 +145,7 @@ extern "C" {
         catch (z3_exception& e) {
             if (owned)
                 ctx = nullptr;
-            errstrm << e.msg();
+            errstrm << e.what();
             SET_ERROR_CODE(Z3_PARSER_ERROR, errstrm.str());
             return of_ast_vector(v);
         }
@@ -177,6 +178,7 @@ extern "C" {
                                        Z3_symbol const decl_names[],
                                        Z3_func_decl const decls[]) {
         Z3_TRY;
+        RESET_ERROR_CODE();
         ast_manager& m = mk_c(c)->m();
         scoped_ptr<cmd_context> ctx = alloc(cmd_context, false, &(m));
         install_dl_cmds(*ctx.get());
@@ -219,6 +221,7 @@ extern "C" {
                                         Z3_symbol const decl_names[],
                                         Z3_func_decl const decls[]) {
         Z3_TRY;
+        RESET_ERROR_CODE();
         LOG_Z3_parse_smtlib2_string(c, file_name, num_sorts, sort_names, sorts, num_decls, decl_names, decls);
         std::ifstream is(file_name);
         if (!is) {
@@ -232,8 +235,9 @@ extern "C" {
 
     Z3_string Z3_API Z3_eval_smtlib2_string(Z3_context c, Z3_string str) {
         std::stringstream ous;
-        Z3_TRY;
+        RESET_ERROR_CODE();
         LOG_Z3_eval_smtlib2_string(c, str);
+        Z3_TRY;
         if (!mk_c(c)->cmd()) {
             auto* ctx = alloc(cmd_context, false, &(mk_c(c)->m()));
             mk_c(c)->cmd() = ctx;
@@ -253,15 +257,13 @@ extern "C" {
             // See api::context::m_parser for a motivation about the reuse of the parser
             if (!parse_smt2_commands_with_parser(mk_c(c)->m_parser, *ctx.get(), is)) {
                 SET_ERROR_CODE(Z3_PARSER_ERROR, ous.str());
-                RETURN_Z3(mk_c(c)->mk_external_string(ous.str()));
             }
         }
         catch (z3_exception& e) {
-            if (ous.str().empty()) ous << e.msg();
+            if (ous.str().empty()) ous << e.what();
             SET_ERROR_CODE(Z3_PARSER_ERROR, ous.str());
-            RETURN_Z3(mk_c(c)->mk_external_string(ous.str()));
         }
-        RETURN_Z3(mk_c(c)->mk_external_string(ous.str()));
-        Z3_CATCH_RETURN(mk_c(c)->mk_external_string(ous.str()));
+        Z3_CATCH_CORE({});
+        RETURN_Z3(mk_c(c)->mk_external_string(std::move(ous).str()));
     }
 }
