@@ -107,34 +107,18 @@ namespace Microsoft.Z3
             Debug.Assert(ctx != null);
         }
 
-        /// <summary>
-        /// DecRefQueue
-        /// </summary>
-        internal class DecRefQueue : IDecRefQueue
-        {
-            public DecRefQueue() : base() { }
-            public DecRefQueue(uint move_limit) : base(move_limit) { }
-            internal override void IncRef(Context ctx, IntPtr obj)
-            {
-                Native.Z3_tactic_inc_ref(ctx.nCtx, obj);
-            }
-
-            internal override void DecRef(Context ctx, IntPtr obj)
-            {
-                Native.Z3_tactic_dec_ref(ctx.nCtx, obj);
-            }
-        };
-
         internal override void IncRef(IntPtr o)
         {
-            Context.Tactic_DRQ.IncAndClear(Context, o);
-            base.IncRef(o);
+            Native.Z3_tactic_inc_ref(Context.nCtx, o);
         }
 
         internal override void DecRef(IntPtr o)
         {
-            Context.Tactic_DRQ.Add(o);
-            base.DecRef(o);
+            lock (Context)
+            {
+                if (Context.nCtx != IntPtr.Zero)
+                    Native.Z3_tactic_dec_ref(Context.nCtx, o);
+            }
         }
         #endregion
     }
