@@ -113,14 +113,21 @@ def _clean_native_build():
 
 def _z3_version():
     post = os.getenv('Z3_VERSION_SUFFIX', '')
+    print("z3_version", "release dir", RELEASE_DIR)   
     if RELEASE_DIR is None:
-        fn = os.path.join(SRC_DIR, 'scripts', 'mk_project.py')
-        if os.path.exists(fn):
-            with open(fn) as f:
-                for line in f:
-                    n = re.match(r".*set_version\((.*), (.*), (.*), (.*)\).*", line)
-                    if not n is None:
-                        return n.group(1) + '.' + n.group(2) + '.' + n.group(3) + '.' + n.group(4) + post
+        dirs = [SRC_DIR, ROOT_DIR, SRC_DIR_REPO, SRC_DIR_LOCAL, os.path.join(ROOT_DIR, '..', '..')]
+        for d in dirs:
+            if os.path.exists(d):
+               print(d, ": ", os.listdir(d))
+        fns = [os.path.join(d, 'scripts', 'VERSION.txt') for d in dirs]        
+        for fn in fns:
+            print("loading version file", fn, "exists", os.path.exists(fn))
+            if os.path.exists(fn):
+                with open(fn) as f:
+                    for line in f:
+                        n = re.match(r"(.*)\.(.*)\.(.*)\.(.*)", line)
+                        if not n is None:
+                            return n.group(1) + '.' + n.group(2) + '.' + n.group(3) + '.' + n.group(4) + post
         return "?.?.?.?"
     else:
         version = RELEASE_METADATA[0]
@@ -284,7 +291,7 @@ class sdist(_sdist):
 # The Azure Dev Ops pipelines use internal OS version tagging that don't correspond
 # to releases.
 
-internal_build_re = re.compile("(.+)\_7")
+internal_build_re = re.compile("(.+)_7")
 
 class bdist_wheel(_bdist_wheel):
 
