@@ -31,6 +31,7 @@ Revision History:
 #include "ast/datatype_decl_plugin.h"
 #include "ast/seq_decl_plugin.h"
 #include "ast/fpa_decl_plugin.h"
+#include "ast/recfun_decl_plugin.h"
 #include "ast/for_each_ast.h"
 #include "ast/decl_collector.h"
 #include "math/polynomial/algebraic_numbers.h"
@@ -986,7 +987,7 @@ void ast_smt_pp::display_smt2(std::ostream& strm, expr* n) {
     ast_mark sort_mark;
     for (sort* s : decls.get_sorts()) {
         if (!(*m_is_declared)(s)) {
-            smt_printer p(strm, m, ql, rn, m_logic, true, true, m_simplify_implies, 0);
+            smt_printer p(strm, m, ql, rn, m_logic, true, m_simplify_implies, 0);
             p.pp_sort_decl(sort_mark, s);
         }
     }
@@ -994,29 +995,41 @@ void ast_smt_pp::display_smt2(std::ostream& strm, expr* n) {
     for (unsigned i = 0; i < decls.get_num_decls(); ++i) {
         func_decl* d = decls.get_func_decls()[i];
         if (!(*m_is_declared)(d)) {
-            smt_printer p(strm, m, ql, rn, m_logic, true, true, m_simplify_implies, 0);
+            smt_printer p(strm, m, ql, rn, m_logic, true, m_simplify_implies, 0);
             p(d);
             strm << "\n";
         }
     }
 
+    vector<std::pair<func_decl*, expr*>> recfuns;
+    recfun::util u(m);
+    for (auto f : decls.get_rec_decls()) 
+        recfuns.push_back({f, u.get_def(f).get_rhs()});
+
+
+    if (!recfuns.empty()) {
+        smt2_pp_environment_dbg env(m);
+        ast_smt2_pp_recdefs(strm, recfuns, env);
+    }
+
+
 #endif
 
     for (expr* a : m_assumptions) {
-        smt_printer p(strm, m, ql, rn, m_logic, false, true, m_simplify_implies, 1);
+        smt_printer p(strm, m, ql, rn, m_logic, false, m_simplify_implies, 1);
         strm << "(assert\n ";
         p(a);
         strm << ")\n";
     }
 
     for (expr* a : m_assumptions_star) {
-        smt_printer p(strm, m, ql, rn, m_logic, false, true, m_simplify_implies, 1);
+        smt_printer p(strm, m, ql, rn, m_logic, false, m_simplify_implies, 1);
         strm << "(assert\n ";
         p(a);
         strm << ")\n";
     }
 
-    smt_printer p(strm, m, ql, rn, m_logic, false, true, m_simplify_implies, 0);
+    smt_printer p(strm, m, ql, rn, m_logic, false, m_simplify_implies, 0);
     if (m.is_bool(n)) {
         if (!m.is_true(n)) {
             strm << "(assert\n ";
