@@ -38,6 +38,10 @@ public:
 
     void updt_params(params_ref const & p);
     static void get_param_descrs(param_descrs & r);
+
+    void set_flat_and_or(bool f);
+    void set_order_eq(bool f);
+
     unsigned get_cache_size() const;
     unsigned get_num_steps() const;
    
@@ -47,6 +51,9 @@ public:
     expr_ref operator()(expr * n, unsigned num_bindings, expr * const * bindings);
 
     expr_ref mk_app(func_decl* f, unsigned num_args, expr* const* args);
+    expr_ref mk_app(func_decl* f, ptr_vector<expr> const& args) { return mk_app(f, args.size(), args.data()); }
+    expr_ref mk_app(func_decl* f, expr_ref_vector const& args) { return mk_app(f, args.size(), args.data()); }
+    expr_ref mk_eq(expr* a, expr* b);
 
     bool reduce_quantifier(quantifier * old_q, 
                            expr * new_body, 
@@ -67,7 +74,6 @@ public:
     expr_dependency * get_used_dependencies();
     void reset_used_dependencies();
 
-    void set_solver(expr_solver* solver);
 
 };
 
