@@ -192,7 +192,7 @@ public:
         labels_vec labels;
         goal_ref g = alloc(goal, m, ctx.produce_proofs(), ctx.produce_models(), ctx.produce_unsat_cores());
         assert_exprs_from(ctx, *g);
-        TRACE("check_sat_using", g->display(tout););
+        TRACE(check_sat_using, g->display(tout););
         model_ref           md;
         proof_ref           pr(m);
         expr_dependency_ref core(m);
@@ -227,8 +227,8 @@ public:
                 }
                 catch (z3_exception & ex) {
                     result->set_status(l_undef);
-                    result->m_unknown = ex.msg();
-                    ctx.regular_stream() << "(error \"tactic failed: " << ex.msg() << "\")" << std::endl;
+                    result->m_unknown = ex.what();
+                    ctx.regular_stream() << "(error \"tactic failed: " << ex.what() << "\")" << std::endl;
                 }
                 ctx.validate_check_sat_result(r);
             }
@@ -321,7 +321,7 @@ public:
                     exec(t, g, result_goals);
                 }
                 catch (tactic_exception & ex) {
-                    ctx.regular_stream() << "(error \"tactic failed: " << ex.msg() << "\")" << std::endl;
+                    ctx.regular_stream() << "(error \"tactic failed: " << ex.what() << "\")" << std::endl;
                     failed = true;
                 }
             }
