@@ -60,22 +60,20 @@ unsigned common::random() {
 }
 
 void common::add_deps_of_fixed(lpvar j, u_dependency*& dep) {
-    unsigned lc, uc;
-    auto& dep_manager = c().m_intervals.get_dep_intervals().dep_manager();
-    c().m_lar_solver.get_bound_constraint_witnesses_for_column(j, lc, uc);
-    dep = dep_manager.mk_join(dep, dep_manager.mk_leaf(lc));
-    dep = dep_manager.mk_join(dep, dep_manager.mk_leaf(uc));                    
+    auto& dm = c().lra.dep_manager();  
+    auto* deps = c().lra.get_bound_constraint_witnesses_for_column(j);
+    dep = dm.mk_join(dep, deps);               
 }
 
 
 // creates a nex expression for the coeff and var, 
 nex * common::nexvar(const rational & coeff, lpvar j, nex_creator& cn, u_dependency*& dep) {
     SASSERT(!coeff.is_zero());
-    if (c().m_nla_settings.horner_subs_fixed() == 1 && c().var_is_fixed(j)) {
+    if (c().params().arith_nl_horner_subs_fixed() == 1 && c().var_is_fixed(j)) {
         add_deps_of_fixed(j, dep);
-        return cn.mk_scalar(coeff * c().m_lar_solver.column_lower_bound(j).x);
+        return cn.mk_scalar(coeff * c().lra.column_lower_bound(j).x);
     }
-    if (c().m_nla_settings.horner_subs_fixed() == 2 && c().var_is_fixed_to_zero(j)) {
+    if (c().params().arith_nl_horner_subs_fixed() == 2 && c().var_is_fixed_to_zero(j)) {
         add_deps_of_fixed(j, dep);
         return cn.mk_scalar(rational(0));
     }
@@ -89,10 +87,10 @@ nex * common::nexvar(const rational & coeff, lpvar j, nex_creator& cn, u_depende
     mf *= coeff;
     u_dependency * initial_dep = dep;
     for (lpvar k : m.vars()) {
-        if (c().m_nla_settings.horner_subs_fixed() && c().var_is_fixed(k)) {
+        if (c().params().arith_nl_horner_subs_fixed() == 1 && c().var_is_fixed(k)) {
             add_deps_of_fixed(k, dep);
-            mf *= c().m_lar_solver.column_lower_bound(k).x;
-        } else if (c().m_nla_settings.horner_subs_fixed() == 2 &&
+            mf *= c().lra.column_lower_bound(k).x;
+        } else if (c().params().arith_nl_horner_subs_fixed() == 2 &&
                    c().var_is_fixed_to_zero(k)) {
             dep = initial_dep;
             add_deps_of_fixed(k, dep);
@@ -101,11 +99,11 @@ nex * common::nexvar(const rational & coeff, lpvar j, nex_creator& cn, u_depende
         else {
             c().insert_to_active_var_set(k);
             mf *= cn.mk_var(k);
-            CTRACE("nla_grobner", c().is_monic_var(k), c().print_var(k, tout) << "\n";);
+            CTRACE(nla_grobner, c().is_monic_var(k), c().print_var(k, tout) << "\n";);
         }
     }
     nex* e = mf.mk();
-    TRACE("nla_grobner", tout << *e;);
+    TRACE(nla_grobner, tout << *e;);
     return e;
 }
 
@@ -115,7 +113,7 @@ template <typename T> void common::create_sum_from_row(const T& row,
                                                        nex_creator::sum_factory& sum,
                                                        u_dependency*& dep) {
 
-    TRACE("nla_horner", tout << "row="; m_core.print_row(row, tout) << "\n";);
+    TRACE(nla_horner, tout << "row="; m_core.print_row(row, tout) << "\n";);
     SASSERT(row.size() > 1);
     sum.reset();
     for (const auto &p : row) {
@@ -129,4 +127,4 @@ template <typename T> void common::create_sum_from_row(const T& row,
 
 
 }
-template void nla::common::create_sum_from_row<vector<lp::row_cell<rational>, true, unsigned int> >(vector<lp::row_cell<rational>, true, unsigned int> const&, nla::nex_creator&, nla::nex_creator::sum_factory&, u_dependency*&);  
+template void nla::common::create_sum_from_row<std_vector<lp::row_cell<rational>> >(std_vector<lp::row_cell<rational>> const&, nla::nex_creator&, nla::nex_creator::sum_factory&, u_dependency*&);  
