@@ -11,6 +11,7 @@ Author:
 
 --*/
 #include "util/uint_set.h"
+#include "params/smt_params_helper.hpp"
 #include "math/lp/nla_core.h"
 #include "math/lp/factorization_factory_imp.h"
 #include "math/grobner/pdd_solver.h"
@@ -27,6 +28,13 @@ namespace nla {
         m_quota(m_core.params().arith_nl_gr_q())
     {}
 
+    void grobner::updt_params(params_ref const& p) {
+        smt_params_helper ph(p);
+        m_config.m_propagate_quotients = ph.arith_nl_grobner_propagate_quotients();
+        m_config.m_gcd_test = ph.arith_nl_grobner_gcd_test();
+        m_config.m_expand_terms = ph.arith_nl_grobner_expand_terms();
+    }
+
     lp::lp_settings& grobner::lp_settings() {
         return c().lp_settings();
     }
@@ -39,14 +47,28 @@ namespace nla {
         if (m_quota == 0)
             m_quota = c().params().arith_nl_gr_q();                    
 
+        bool const use_exp_delay = c().params().arith_nl_grobner_exp_delay();
+
         if (m_quota == 1) {
-            m_delay_base++;
-            m_delay = m_delay_base;
+            if (use_exp_delay) {
+                constexpr unsigned delay_cap = 1000000;
+                if (m_delay_base == 0)
+                    m_delay_base = 1;
+                else if (m_delay_base < delay_cap) {
+                    m_delay_base *= 2;
+                    if (m_delay_base > delay_cap)
+                        m_delay_base = delay_cap;
+                }
+                m_delay = m_delay_base;
+            }
+            else
+                m_delay = ++m_delay_base;
             m_quota = c().params().arith_nl_gr_q();
         }
 
         if (m_delay > 0) {
             --m_delay;
+            TRACE(grobner, tout << "delay " << m_delay << "\n");
             return;
         }
         
@@ -55,6 +77,7 @@ namespace nla {
         if (!configure())
             return;
         m_solver.saturate();
+        TRACE(grobner, m_solver.display(tout));
 
         if (m_delay_base > 0)
             --m_delay_base;
@@ -64,12 +87,18 @@ namespace nla {
             if (is_conflicting())
                 return;
 
+            if (propagate_quotients())
+                return;
+
+            if (propagate_gcd_test())
+                return;
+
             if (propagate_eqs())
                 return;
-            
+
             if (propagate_factorization())
                 return;
-            
+
             if (propagate_linear_equations())
                 return;
             
@@ -90,14 +119,6 @@ namespace nla {
         IF_VERBOSE(5, diagnose_pdd_miss(verbose_stream()));
     }
 
-    dd::solver::equation_vector const& grobner::core_equations(bool all_eqs) {
-        flet<bool> _add_all(m_add_all_eqs, all_eqs);
-        find_nl_cluster();        
-        if (!configure()) 
-            throw dd::pdd_manager::mem_out();
-        return m_solver.equations();
-    }
-
     bool grobner::is_conflicting() {
         for (auto eq : m_solver.equations()) {
             if (is_conflicting(*eq)) {
@@ -142,9 +163,10 @@ namespace nla {
             ineq new_eq(v, llc::EQ, rational::zero());
             if (c().ineq_holds(new_eq))
                 return false;
-            lemma_builder lemma(c(), "pdd-eq");
+            lemma_builder lemma(c(), "grobner-eq");
             add_dependencies(lemma, eq);
             lemma |= new_eq;
+            TRACE(grobner, lemma.display(tout););
             return true;
         }
         if (p.is_offset()) {
@@ -159,9 +181,10 @@ namespace nla {
             ineq new_eq(term(a, v), llc::EQ, b);
             if (c().ineq_holds(new_eq))
                 return false;
-            lemma_builder lemma(c(), "pdd-eq");
+            lemma_builder lemma(c(), "grobner-eq");
             add_dependencies(lemma, eq);
             lemma |= new_eq;
+            TRACE(grobner, lemma.display(tout););
             return true;
         }
 
@@ -181,9 +204,30 @@ namespace nla {
 
         // IF_VERBOSE(0, verbose_stream() << "factored " << q << " : " << vars << "\n");
 
-        term t;
+        auto [t, offset] = linear_to_term(q);
+
+        vector<ineq> ineqs;
+        for (auto v : vars)
+            ineqs.push_back(ineq(v, llc::EQ, rational::zero()));
+        ineqs.push_back(ineq(t, llc::EQ, -offset));
+        for (auto const& i : ineqs)
+            if (c().ineq_holds(i))
+                return false;
+
+        lemma_builder lemma(c(), "grobner-factored");
+        add_dependencies(lemma, eq);
+        for (auto const& i : ineqs)
+            lemma |= i;
+        //lemma.display(verbose_stream());
+        return true;
+    }
+
+
+    std::pair<lp::lar_term, rational> grobner::linear_to_term(dd::pdd q) {
+        SASSERT(q.is_linear());
         rational lc(1);
         auto ql = q;
+        lp::lar_term t;
         while (!ql.is_val()) {
             lc = lcm(lc, denominator(ql.hi().val()));
             ql = ql.lo();
@@ -191,23 +235,281 @@ namespace nla {
         lc = lcm(denominator(ql.val()), lc);
 
         while (!q.is_val()) {
-            t.add_monomial(lc*q.hi().val(), q.var());
+            t.add_monomial(lc * q.hi().val(), q.var());
             q = q.lo();
         }
-        vector<ineq> ineqs;
-        for (auto v : vars)
-            ineqs.push_back(ineq(v, llc::EQ, rational::zero()));
-        ineqs.push_back(ineq(t, llc::EQ, -lc*q.val()));
-        for (auto const& i : ineqs)
-            if (c().ineq_holds(i))
-                return false;
+        rational offset = lc * q.val();
+        return {t, offset};
+    }
 
-        lemma_builder lemma(c(), "pdd-factored");
-        add_dependencies(lemma, eq);
-        for (auto const& i : ineqs)
-            lemma |= i;
-        //lemma.display(verbose_stream());
-        return true;
+    bool grobner::propagate_gcd_test() {
+        if (!m_config.m_gcd_test)
+            return false;
+        unsigned changed = 0;
+        for (auto eq : m_solver.equations())
+            if (propagate_gcd_test(*eq) && ++changed >= m_solver.number_of_conflicts_to_report())
+                return true;
+        return changed > 0;
+    }
+
+    // x^2 - y = 0 => y mod 4 = { 0, 1 }
+    // x^k - y = 0 => y mod k^2 = {0, 1, .., (k-1)^k }
+    bool grobner::propagate_gcd_test(dd::solver::equation const& eq) {
+        dd::pdd p = eq.poly();
+        if (p.is_linear())
+            return false;
+        if (p.is_val())
+            return false;
+        auto v = p.var();
+        if (!c().var_is_int(v))
+            return false;
+        for (auto v : p.free_vars())
+            if (!c().var_is_int(v))
+                return false;
+        dd::pdd_eval eval;
+        eval.var2val() = [&](unsigned j) { return val(j); };
+        if (eval(p) == 0)
+            return false;
+        p.get_powers(m_powers);
+        if (m_powers.empty())
+            return false;
+        rational d(1);
+        for (auto const& value : p.coefficients())
+            d = lcm(d, denominator(value.coeff));
+        if (d != 1)
+            p *= d;
+        auto& m = p.manager();
+        auto fails_gcd_test = [&](dd::pdd const& q) {
+            rational g(0);
+            rational k(0);
+            for (auto const& value : q.coefficients()) {
+                if (value.is_constant) {
+                    k += value.coeff;
+                    continue;
+                }
+                g = gcd(g, value.coeff);
+                if (g == 1)
+                    return false;                
+            }
+            return k != 0 && !divides(g, k);
+        };
+        for (auto [x, k] : m_powers) {
+            SASSERT(k > 1);
+            if (k > 4)          // cut off at larger exponents.
+                continue;
+            bool gcd_fail = true;
+            dd::pdd kx = m.mk_var(x) * m.mk_val(k); 
+            for (unsigned r = 0; gcd_fail && r < k; r++) {
+                dd::pdd kx_plus_r = kx + m.mk_val(r);
+                auto q = p.subst_pdd(x, kx_plus_r);
+                if (!fails_gcd_test(q))
+                    gcd_fail = false;
+            }
+            if (gcd_fail) {
+                lemma_builder lemma(c(), "grobner-gcd-test");
+                add_dependencies(lemma, eq);
+                return true;
+            }
+        }
+        return false;
+    }
+
+    bool grobner::propagate_quotients() {
+        if (!m_config.m_propagate_quotients)
+            return false;
+        unsigned changed = 0;
+        for (auto eq : m_solver.equations())
+            if (propagate_quotients(*eq) && ++changed >= m_solver.number_of_conflicts_to_report())
+                return true;
+        return changed > 0;
+    }
+
+    // factor each nl var at a time.
+    // x*y + z = 0
+    // x = 0 => z = 0
+    // y = 0 => z = 0
+    // z = 0 => x = 0 or y = 0
+    // z > 0 & x > 0 => x <= z
+    // z < 0 & x > 0 => x <= -z
+    // z > 0 & x < 0 => -x <= z
+    // z < 0 & x < 0 => -x <= -z
+    bool grobner::propagate_quotients(dd::solver::equation const& eq) {
+        dd::pdd const& p = eq.poly();
+        if (p.is_linear())
+            return false;
+        if (p.is_val())
+            return false;
+        auto v = p.var();
+        if (!c().var_is_int(v))
+            return false;
+        for (auto v : p.free_vars())
+            if (!c().var_is_int(v))
+                return false;
+        dd::pdd_eval eval;
+        eval.var2val() = [&](unsigned j) { return val(j); };
+        if (eval(p) == 0)
+            return false;
+        TRACE(grobner, tout << "propagate_quotients " << p << "\n");
+        tracked_uint_set nl_vars;
+        rational d(1);
+        for (auto const& m : p) {
+            d = lcm(d, denominator(m.coeff));
+            if (m.vars.size() == 1)
+                continue;
+            for (auto j : m.vars)
+                nl_vars.insert(j);
+        }
+
+        bool found_lemma = false;
+        for (auto v : nl_vars) {
+            auto& m = p.manager();
+            dd::pdd lc(m), r(m);
+            p.factor(v, 1, lc, r);
+            if (!r.is_linear())
+                continue;
+            if (d != 1) {
+                lc *= d;
+                r *= d;
+            }
+            auto v_value = val(v);
+            auto r_value = eval(r);
+            auto lc_value = eval(lc);
+
+            TRACE(grobner, tout << "nl-var: " << v << " " << v_value << "\n"
+                                << "lc: " << lc << " " << lc_value << "\n"
+                                << "r: " << r << " " << r_value << "\n";
+            );
+            SASSERT(v_value.is_int());
+            SASSERT(r_value.is_int());
+            SASSERT(lc_value.is_int());
+            if (r_value == 0) {
+                if (v_value == 0)
+                    continue;
+                if (lc_value == 0)
+                    continue;
+                if (!lc.is_linear())
+                    continue;
+                auto [t, offset] = linear_to_term(lc);
+                auto [t2, offset2] = linear_to_term(r);
+                lemma_builder lemma(c(), "grobner-quotient");
+                add_dependencies(lemma, eq);
+                // v = 0 or lc = 0 or r != 0
+                lemma |= ineq(v, llc::EQ, rational::zero());
+                lemma |= ineq(t, llc::EQ, -offset);
+                lemma |= ineq(t2, llc::NE, -offset2);
+                TRACE(grobner, lemma.display(tout << "quotient1\n"));
+                found_lemma = true;
+                continue;
+            }
+            // r_value != 0
+            if (v_value == 0) {
+                // v = 0 => r = 0
+                lemma_builder lemma(c(), "grobner-quotient");
+                add_dependencies(lemma, eq);
+                auto [t, offset] = linear_to_term(r);
+                lemma |= ineq(v, llc::NE, rational::zero());
+                lemma |= ineq(t, llc::EQ, -offset);
+                TRACE(grobner, lemma.display(tout << "quotient2\n"));
+                found_lemma = true;
+                continue;
+            }
+            if (lc_value == 0) {
+                if (!lc.is_linear())
+                    continue;
+                // lc = 0 => r = 0
+                lemma_builder lemma(c(), "grobner-quotient");
+                add_dependencies(lemma, eq);
+                auto [t, offset] = linear_to_term(lc);
+                auto [t2, offset2] = linear_to_term(r);
+                lemma |= ineq(t, llc::NE, -offset);
+                lemma |= ineq(t2, llc::EQ, -offset2);
+                TRACE(grobner, lemma.display(tout << "quotient3\n"));
+                found_lemma = true;
+                continue;           
+            }
+            if (divides(v_value, r_value))
+                continue;
+              
+            if (abs(v_value) > abs(r_value)) {
+                // v*c + r = 0 & v > 0 => r >= v or -r >= v or r = 0
+                lemma_builder lemma(c(), "grobner-quotient");
+                auto [t, offset] = linear_to_term(r);
+                add_dependencies(lemma, eq);
+                if (v_value > 0 && r_value > 0) {
+                    // v*c + t = 0 => v <= 0 or v <= t or t <= 0
+                    lemma |= ineq(v, llc::LE, rational::zero()); // v <= 0
+                    lemma |= ineq(t, llc::LE, -offset);          // t <= 0
+                    t.add_monomial(rational(-1), v);
+                    lemma |= ineq(t, llc::GE, -offset);          // t - v >= 0
+                } 
+                else if (v_value > 0 && r_value < 0) {
+                    // v*c + t = 0 => v <= 0 or v <= -t or t >= 0
+                    lemma |= ineq(v, llc::LE, rational::zero());  // v <= 0
+                    lemma |= ineq(t, llc::GE, -offset);           // t >= 0
+                    t.add_monomial(rational(1), v);
+                    lemma |= ineq(t, llc::LE, -offset);           // t + v <= 0
+                } 
+                else if (v_value < 0 && r_value > 0) {
+                    // v*c + t = 0 => v >= 0 or -v <= t or t <= 0                     
+                    lemma |= ineq(v, llc::GE, rational::zero());  // v >= 0
+                    lemma |= ineq(t, llc::LE, -offset);           // t <= 0
+                    t.add_monomial(rational(1), v);
+                    lemma |= ineq(t, llc::GE, -offset);           // t + v >= 0
+                } 
+                else if (v_value < 0 && r_value < 0) {
+                    // v*c + t = 0 => v >= 0 or v >= t or t >= 0 
+                    lemma |= ineq(v, llc::GE, rational::zero());  // v >= 0
+                    lemma |= ineq(t, llc::GE, -offset);           // t >= 0
+                    t.add_monomial(rational(-1), v);
+                    lemma |= ineq(t, llc::LE, -offset);           // t - v <= 0
+                }
+                TRACE(grobner, lemma.display(tout << "quotient4\n"));
+                found_lemma = true;
+                continue;
+            } 
+            if (abs(lc_value) > 1 && abs(v_value) > 1 && abs(v_value) >= abs(r_value)) {
+                // v*c + t = 0 & |c| > 1, |v| > 1 => |v| < |t|
+                lemma_builder lemma(c(), "grobner-bounds");
+                auto [tr, offset_t] = linear_to_term(r);
+                auto [tc, offset_c] = linear_to_term(lc);
+                add_dependencies(lemma, eq);
+                if (lc_value > 1) // c <= 1                   
+                    lemma |= ineq(tc, llc::LE, rational(1) - offset_c);  // lc <= 1                
+                else // c >= -1
+                    lemma |= ineq(tc, llc::GE, rational(-1) - offset_c);  // lc >= -1
+                if (v_value > 1)                                          
+                    lemma |= ineq(v, llc::LE, rational(1));               // v <= 1
+                else                                                      
+                    lemma |= ineq(v, llc::GE, rational(-1));              // v >= -1
+                if (v_value > 0 && r_value > 0) {
+                    // t >= v + 1
+                    tr.add_monomial(rational(-1), v);
+                    lemma |= ineq(tr, llc::GE, rational(1) - offset_t); 
+                }
+                else if (v_value > 0 && r_value < 0) {
+                    // t + v <= -1
+                    tr.add_monomial(rational(1), v);
+                    lemma |= ineq(tr, llc::LE, rational(-1) - offset_t); 
+                }
+                else if (v_value < 0 && r_value > 0) {
+                    // t + v >= 1
+                    tr.add_monomial(rational(1), v);
+                    lemma |= ineq(tr, llc::GE, rational(1) - offset_t); 
+                }
+                else if (v_value < 0 && r_value < 0) {
+                    // t - v <= -1
+                    tr.add_monomial(rational(-1), v);
+                    lemma |= ineq(tr, llc::LE, rational(-1) - offset_t); 
+                }
+                found_lemma = true;
+                TRACE(grobner, lemma.display(tout << "quotient5\n"));
+                continue;
+            }
+            // other division lemmas are possible.  
+            // also extend to non-linear r, non-linear lc         
+        }
+        CTRACE(grobner, !found_lemma, tout << "no lemmas found for " << p << "\n");
+        return found_lemma;
     }
 
     void grobner::explain(dd::solver::equation const& eq, lp::explanation& exp) {
@@ -322,24 +624,11 @@ namespace nla {
         };
         scoped_dep_interval i(di), i_wd(di);
         evali.get_interval<dd::w_dep::without_deps>(e.poly(), i);    
-        if (!di.separated_from_zero(i)) {
-            TRACE(grobner, m_solver.display(tout << "not separated from 0 ", e) << "\n";
-                  evali.get_interval_distributed<dd::w_dep::without_deps>(e.poly(), i);
-                  tout << "separated from 0: " << di.separated_from_zero(i) << "\n";
-                  for (auto j : e.poly().free_vars()) {
-                      scoped_dep_interval a(di);
-                      c().m_intervals.set_var_interval<dd::w_dep::without_deps>(j, a);
-                      c().m_intervals.display(tout << "j" << j << " ", a); tout << " ";
-                  }
-                  tout << "\n");
-
-            
-            if (add_horner_conflict(e))
+        if (!di.separated_from_zero(i)) {            
+            if (add_horner_conflict(e)) {
+                TRACE(grobner, m_solver.display(tout << "horner conflict ", e) << "\n");
                 return true;
-#if 0
-            if (add_nla_conflict(e)) 
-                return true;
-#endif
+            }
             return false;
         }
         evali.get_interval<dd::w_dep::with_deps>(e.poly(), i_wd);  
@@ -352,10 +641,6 @@ namespace nla {
             return true;
         }
         else {
-#if 0
-            if (add_nla_conflict(e)) 
-                return true;
-#endif
             TRACE(grobner, m_solver.display(tout << "no conflict ", e) << "\n");
             return false;
         }
@@ -423,8 +708,12 @@ namespace nla {
                 for (const factor& fc: fcn) 
                     q.push_back(var(fc));        
         }
-
-        if (c().var_is_fixed(j))
+        else if (c().lra.column_has_term(j)) {
+            const lp::lar_term& t = c().lra.get_term(j);
+            for (auto k : t)
+                q.push_back(k.j());
+        }
+        else if (c().var_is_fixed(j))
             return;
         const auto& matrix = lra.A_r();
         for (auto & s : matrix.m_columns[j]) {
@@ -433,12 +722,18 @@ namespace nla {
                 continue;
             m_rows.insert(row);
             unsigned k = lra.get_base_column_in_row(row);
-            // grobner bassis does not know about integer constraints
-            if (lra.column_is_free(k) && !m_add_all_eqs && k != j)
-                continue;
             // a free column over the reals can be assigned
-            if (lra.column_is_free(k) && k != j && !lra.var_is_int(k)) 
-                continue;
+            if (lra.column_is_free(k) && k != j) {
+                if (!lra.var_is_int(k))
+                    continue;
+                // free integer columns are ignored unless m_add_all_eqs is set or we are doing gcd test.
+                if (!m_add_all_eqs && !m_config.m_gcd_test && !m_config.m_propagate_quotients)
+                    continue;
+                // a free integer column with integer coefficients can be assigned.
+                if (!m_add_all_eqs && all_of(c().lra.get_row(row), [&](auto& ri) { return ri.coeff().is_int();}))
+                    continue;
+            }
+
             CTRACE(grobner, matrix.m_rows[row].size() > c().params().arith_nl_grobner_row_length_limit(),
                    tout << "ignore the row " << row << " with the size " << matrix.m_rows[row].size() << "\n";);
             // limits overhead of grobner equations, unless this is for extracting a complete COI of the non-satisfied subset.
@@ -456,6 +751,13 @@ namespace nla {
         return lra.column_lower_bound(j).x;
     }
 
+    dd::pdd grobner::pdd_expr(lp::lar_term const& t, u_dependency*& dep) {
+        dd::pdd r = m_pdd_manager.mk_val(0);
+        for (auto const& ti : t) 
+            r += pdd_expr(ti.coeff(), ti.j(), dep);        
+        return r;
+    }
+
     dd::pdd grobner::pdd_expr(const rational& coeff, lpvar j, u_dependency*& dep) {
         dd::pdd r = m_pdd_manager.mk_val(coeff);
         sbuffer<lpvar> vars;
@@ -464,6 +766,7 @@ namespace nla {
         while (!vars.empty()) {
             j = vars.back();
             vars.pop_back();
+
             if (c().params().arith_nl_grobner_subs_fixed() > 0 && c().var_is_fixed_to_zero(j)) {
                 r = m_pdd_manager.mk_val(val_of_fixed_var_with_deps(j, zero_dep));
                 dep = zero_dep;
@@ -471,6 +774,8 @@ namespace nla {
             }
             if (c().params().arith_nl_grobner_subs_fixed() == 1 && c().var_is_fixed(j))
                 r *= val_of_fixed_var_with_deps(j, dep);
+            else if (m_config.m_expand_terms && c().lra.column_has_term(j))
+                r *= pdd_expr(c().lra.get_term(j), dep);
             else if (!c().is_monic_var(j))
                 r *= m_pdd_manager.mk_var(j);
             else
@@ -572,7 +877,7 @@ namespace nla {
         prepare_rows_and_active_vars();
         svector<lpvar> q;
         TRACE(grobner, for (lpvar j : c().m_to_refine) print_monic(c().emons()[j], tout) << "\n";);
-          
+         
         for (lpvar j : c().m_to_refine)
             q.push_back(j);
     
@@ -678,7 +983,9 @@ namespace nla {
         cross_nested cn(
             [this, dep](const nex* n) { return c().m_intervals.check_nex(n, dep);  },
             [this](unsigned j)   { return c().var_is_fixed(j); },
-            [this]() { return c().random(); }, nc);
+            c().reslim(),
+            c().random(),
+            nc);
         cn.run(to_sum(e));
         bool ret = cn.done();
         return ret;
