@@ -110,8 +110,6 @@ public:
     bool is_false(expr_ref_vector const& ts);
     bool are_equal(expr* s, expr* t);
     void reset_eval_cache();
-    bool has_solver(); 
-    void set_solver(expr_solver* solver);
     void add_rec_funs();
 
     class scoped_model_completion {
