@@ -3,28 +3,22 @@ Copyright (c) 2011 Microsoft Corporation
 
 Module Name:
 
-    theory_seq_empty.h
-
-Abstract:
-
-    <abstract>
+    seq_factory.h
 
 Author:
 
     Nikolaj Bjorner (nbjorner) 2011-14-11
 
-Revision History:
-
 --*/
 #pragma once
 
 #include "ast/seq_decl_plugin.h"
 #include "model/model_core.h"
+#include "model/value_factory.h"
 
 class seq_factory : public value_factory {
     typedef hashtable<symbol, symbol_hash_proc, symbol_eq_proc> symbol_set;
     model_core& m_model;
-    ast_manager& m;
     seq_util     u;
     symbol_set   m_strings;
     unsigned     m_next;
@@ -36,7 +30,6 @@ public:
     seq_factory(ast_manager & m, family_id fid, model_core& md):
         value_factory(m, fid),
         m_model(md),
-        m(m),
         u(m),
         m_next(0),
         m_unique_delim("!"),
@@ -58,24 +51,24 @@ public:
     // generic method for setting unique sequences
     void set_prefix(expr* uniq) {
         m_trail.push_back(uniq);
-        m_unique_sequences.insert(m.get_sort(uniq), uniq);
+        m_unique_sequences.insert(uniq->get_sort(), uniq);
     }
     
     expr* get_some_value(sort* s) override {
-        if (u.is_seq(s)) {
-            return u.str.mk_empty(s);
-        }
         sort* seq = nullptr;
-        if (u.is_re(s, seq)) {
+        if (u.is_seq(s)) 
+            return u.str.mk_empty(s);
+        if (u.is_re(s, seq)) 
             return u.re.mk_to_re(u.str.mk_empty(seq));
-        }
+        if (u.is_char(s))
+            return u.mk_char('A');
         UNREACHABLE();
         return nullptr;
     }
     bool get_some_values(sort* s, expr_ref& v1, expr_ref& v2) override {
         if (u.is_string(s)) {
-            v1 = u.str.mk_string(symbol("a"));
-            v2 = u.str.mk_string(symbol("b"));
+            v1 = u.str.mk_string("a");
+            v2 = u.str.mk_string("b");
             return true;
         }
         sort* ch;
@@ -89,7 +82,11 @@ public:
                 return false;
             }
         }
-        NOT_IMPLEMENTED_YET();
+        if (u.is_char(s)) {
+            v1 = u.mk_char('a');
+            v2 = u.mk_char('b');
+            return true;
+        }
         return false; 
     }
     expr* get_fresh_value(sort* s) override {
@@ -97,10 +94,11 @@ public:
             while (true) {
                 std::ostringstream strm;
                 strm << m_unique_delim << std::hex << m_next++ << std::dec << m_unique_delim;
-                symbol sym(strm.str());
+                std::string s(strm.str());
+                symbol sym(s);
                 if (m_strings.contains(sym)) continue;
                 m_strings.insert(sym);
-                return u.str.mk_string(sym);
+                return u.str.mk_string(s);
             }
         }
         sort* seq = nullptr, *ch = nullptr;
@@ -109,9 +107,7 @@ public:
             return u.re.mk_to_re(v0);
         }
         if (u.is_char(s)) {
-            //char s[2] = { ++m_char, 0 };
-            //return u.str.mk_char(zstring(s), 0);
-            return u.str.mk_char(zstring("a"), 0);
+            return u.mk_char('a');
         }
         if (u.is_seq(s, ch)) {
             expr* v = m_model.get_fresh_value(ch);
@@ -136,26 +132,22 @@ public:
         return nullptr;
     }
     void register_value(expr* n) override {
-        symbol sym;
-        if (u.str.is_string(n, sym)) {
+        zstring s;
+        if (u.str.is_string(n, s)) {
+            symbol sym(s.encode());
             m_strings.insert(sym);
-            if (sym.str().find(m_unique_delim) != std::string::npos) {
+            if (sym.str().find(m_unique_delim) != std::string::npos) 
                 add_new_delim();
-            }
         }
     }
+
 private:
     
     void add_new_delim() {
-        bool found = true;
-        while (found) {
-            found = false;
-            m_unique_delim += "!";
-            symbol_set::iterator it = m_strings.begin(), end = m_strings.end();
-            for (; it != end && !found; ++it) {
-                found = it->str().find(m_unique_delim) != std::string::npos;                    
-            }
-        }
+    try_again:
+        m_unique_delim += "!";
+        for (auto const& s : m_strings) 
+            if (s.str().find(m_unique_delim) != std::string::npos)
+                goto try_again;
     }
 };
-
