@@ -34,7 +34,7 @@ public:
     typedef obj_map<func_decl, unsigned> partition_map;
 
     /**
-       \Brief Model Based Cartesian projection of lits
+       \brief Model Based Cartesian projection of lits
      */
     void operator()(const partition_map &pmap, expr_ref_vector &lits, model &mdl,
                     vector<expr_ref_vector> &res);
