@@ -46,7 +46,7 @@ void model_evaluator_array_util::eval_exprs(model& mdl, expr_ref_vector& es) {
 bool model_evaluator_array_util::extract_array_func_interp(model& mdl, expr* a, vector<expr_ref_vector>& stores, expr_ref& else_case) {
     SASSERT(m_array.is_array(a));
 
-    TRACE("model_evaluator", tout << mk_pp(a, m) << "\n";);
+    TRACE(model_evaluator, tout << mk_pp(a, m) << "\n";);
     while (m_array.is_store(a)) {
         expr_ref_vector store(m);
         store.append(to_app(a)->get_num_args()-1, to_app(a)->get_args()+1);
@@ -72,7 +72,7 @@ bool model_evaluator_array_util::extract_array_func_interp(model& mdl, expr* a,
             store.push_back(fe->get_result());
             for (unsigned j = 0; j < store.size(); ++j) {
                 if (!is_ground(store[j].get())) {
-                    TRACE("model_evaluator", tout << "could not extract array interpretation: " << mk_pp(a, m) << "\n" << mk_pp(store[j].get(), m) << "\n";);
+                    TRACE(model_evaluator, tout << "could not extract array interpretation: " << mk_pp(a, m) << "\n" << mk_pp(store[j].get(), m) << "\n";);
                     return false;
                 }
             }
@@ -81,11 +81,11 @@ bool model_evaluator_array_util::extract_array_func_interp(model& mdl, expr* a,
         }
         else_case = g->get_else();
         if (!else_case) {
-            TRACE("model_evaluator", tout << "no else case " << mk_pp(a, m) << "\n";);
+            TRACE(model_evaluator, tout << "no else case " << mk_pp(a, m) << "\n";);
             return false;
         }
         if (!is_ground(else_case)) {
-            TRACE("model_evaluator", tout << "non-ground else case " << mk_pp(a, m) << "\n" << mk_pp(else_case, m) << "\n";);
+            TRACE(model_evaluator, tout << "non-ground else case " << mk_pp(a, m) << "\n" << mk_pp(else_case, m) << "\n";);
             return false;
         }
         if (m_array.is_as_array(else_case)) {
@@ -93,16 +93,16 @@ bool model_evaluator_array_util::extract_array_func_interp(model& mdl, expr* a,
             eval(mdl, else_case, r);
             else_case = r;
         }
-        TRACE("model_evaluator", tout << "else case: " << mk_pp(else_case, m) << "\n";);
+        TRACE(model_evaluator, tout << "else case: " << mk_pp(else_case, m) << "\n";);
         return true;
     }
-    TRACE("model_evaluator", tout << "no translation: " << mk_pp(a, m) << "\n";);
+    TRACE(model_evaluator, tout << "no translation: " << mk_pp(a, m) << "\n";);
 
     return false;
 }
 
 void model_evaluator_array_util::eval_array_eq(model& mdl, app* e, expr* arg1, expr* arg2, expr_ref& res) {
-    TRACE("model_evaluator", tout << "array equality: " << mk_pp(e, m) << "\n";);
+    TRACE(model_evaluator, tout << "array equality: " << mk_pp(e, m) << "\n";);
     expr_ref v1(m), v2(m);
     eval (mdl, arg1, v1);
     eval (mdl, arg2, v2);
@@ -114,7 +114,7 @@ void model_evaluator_array_util::eval_array_eq(model& mdl, app* e, expr* arg1, e
     sort* r = get_array_range(s);
     // give up evaluating finite domain/range arrays
     if (!r->is_infinite() && !r->is_very_big() && !s->is_infinite() && !s->is_very_big()) {
-        TRACE("model_evaluator", tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
+        TRACE(model_evaluator, tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
         res.reset ();
         return;
     }
@@ -122,14 +122,14 @@ void model_evaluator_array_util::eval_array_eq(model& mdl, app* e, expr* arg1, e
     expr_ref else1(m), else2(m);
     if (!extract_array_func_interp(mdl, v1, store, else1) ||
             !extract_array_func_interp(mdl, v2, store, else2)) {
-        TRACE("model_evaluator", tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
+        TRACE(model_evaluator, tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
         res.reset ();
         return;
     }
 
     if (else1 != else2) {
         if (m.is_value(else1) && m.is_value(else2)) {
-            TRACE("model_evaluator", tout
+            TRACE(model_evaluator, tout
                     << "defaults are different: " << mk_pp(e, m) << " "
                     << mk_pp(else1, m) << " " << mk_pp(else2, m) << "\n";);
             res = m.mk_false ();
@@ -138,7 +138,7 @@ void model_evaluator_array_util::eval_array_eq(model& mdl, app* e, expr* arg1, e
             eval_array_eq(mdl, e, else1, else2, res);
         }
         else {
-            TRACE("model_evaluator", tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
+            TRACE(model_evaluator, tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
             res.reset ();
         }
         return;
@@ -161,7 +161,7 @@ void model_evaluator_array_util::eval_array_eq(model& mdl, app* e, expr* arg1, e
             continue;
         }
         if (m.is_value(w1) && m.is_value(w2)) {
-            TRACE("model_evaluator", tout << "Equality evaluation: " << mk_pp(e, m) << "\n";
+            TRACE(model_evaluator, tout << "Equality evaluation: " << mk_pp(e, m) << "\n";
                     tout << mk_pp(s1, m) << " |-> " << mk_pp(w1, m) << "\n";
                     tout << mk_pp(s2, m) << " |-> " << mk_pp(w2, m) << "\n";);
             res = m.mk_false ();
@@ -173,7 +173,7 @@ void model_evaluator_array_util::eval_array_eq(model& mdl, app* e, expr* arg1, e
             }
         }
         else {
-            TRACE("model_evaluator", tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
+            TRACE(model_evaluator, tout << "equality is unknown: " << mk_pp(e, m) << "\n";);
             res.reset ();
         }
         return;
