@@ -41,9 +41,6 @@ namespace datalog {
         m_cnst2var(m) {        
     }
 
-    mk_quantifier_instantiation::~mk_quantifier_instantiation() {        
-    }
-
     void mk_quantifier_instantiation::extract_quantifiers(rule& r, expr_ref_vector& conjs, quantifier_ref_vector& qs) {
         conjs.reset();
         qs.reset();
@@ -93,7 +90,7 @@ namespace datalog {
     }
 
     void mk_quantifier_instantiation::match(unsigned i, app* pat, unsigned j, term_pairs& todo, quantifier* q, expr_ref_vector& conjs) {
-        TRACE("dl", tout << "match" << mk_pp(pat, m) << "\n";);
+        TRACE(dl, tout << "match" << mk_pp(pat, m) << "\n";);
         while (j < todo.size()) {
             expr* p = todo[j].first;
             expr* t = todo[j].second;
@@ -160,7 +157,7 @@ namespace datalog {
         m_binding.reverse();
         m_cnst2var(res);
         conjs.push_back(res);
-        TRACE("dl", tout << mk_pp(q, m) << "\n==>\n" << mk_pp(res, m) << "\n";);
+        TRACE(dl, tout << mk_pp(q, m) << "\n==>\n" << mk_pp(res, m) << "\n";);
     }
 
     void mk_quantifier_instantiation::collect_egraph(expr* e) {
@@ -226,7 +223,7 @@ namespace datalog {
 
         fml = m.mk_and(conjs.size(), conjs.data());
         fml = m.mk_implies(fml, r.get_head());
-        TRACE("dl", r.display(m_ctx, tout); tout << mk_pp(fml, m) << "\n";);
+        TRACE(dl, r.display(m_ctx, tout); tout << mk_pp(fml, m) << "\n";);
         
         rule_set added_rules(m_ctx);
         proof_ref pr(m); 
