@@ -44,6 +44,7 @@ namespace nlsat {
         bool                    m_full_dimensional;
         bool                    m_minimize_cores;
         bool                    m_factor;
+        bool                    m_add_all_coeffs;
         bool                    m_signed_project;
         bool                    m_cell_sample;
 
@@ -154,6 +155,7 @@ namespace nlsat {
             m_simplify_cores   = false;
             m_full_dimensional = false;
             m_minimize_cores   = false;
+            m_add_all_coeffs   = true;
             m_signed_project   = false;
         }
 
@@ -622,6 +624,8 @@ namespace nlsat {
 //"An improved projection operation for cylindrical algebraic decomposition of three-dimensional space", by McCallum, Scott
             
         bool is_square_free(polynomial_ref_vector &ps, var x) {
+            if (m_add_all_coeffs)
+                return false;
             polynomial_ref p(m_pm);
             polynomial_ref lc_poly(m_pm);
             polynomial_ref disc_poly(m_pm); 
@@ -1226,6 +1230,7 @@ namespace nlsat {
          * https://arxiv.org/abs/2003.00409 
          */
         void project_cdcac(polynomial_ref_vector & ps, var max_x) {
+            bool first = true;
             if (ps.empty())
                 return;
 
@@ -1244,8 +1249,6 @@ namespace nlsat {
             // Remark: after vanishing coefficients are eliminated, ps may not contain max_x anymore
             
             polynomial_ref_vector samples(m_pm);
-
-            
             if (x < max_x)
                 cac_add_cell_lits(ps, x, samples);
 
@@ -1256,9 +1259,18 @@ namespace nlsat {
                 }
                 TRACE(nlsat_explain, tout << "project loop, processing var "; display_var(tout, x); tout << "\npolynomials\n";
                       display(tout, ps); tout << "\n";);
-                add_lcs(ps, x);
-                psc_discriminant(ps, x);
-                psc_resultant(ps, x);
+                if (first) { // The first run is special because x is not constrained by the sample, we cannot surround it by the root functions.
+                    // we make the polynomials in ps delinable
+                    add_lcs(ps, x);
+                    psc_discriminant(ps, x);
+                    psc_resultant(ps, x);
+                    first = false;
+                }
+                else {
+                    add_lcs(ps, x);
+                    psc_discriminant(ps, x);
+                    psc_resultant_sample(ps, x, samples);
+                }
                 
                 if (m_todo.empty())
                     break;
@@ -2127,6 +2139,10 @@ namespace nlsat {
         m_imp->m_factor = f;
     }
 
+    void explain::set_add_all_coeffs(bool f) {
+        m_imp->m_add_all_coeffs = f;
+    }
+
     void explain::set_signed_project(bool f) {
         m_imp->m_signed_project = f;
     }
@@ -2177,4 +2193,3 @@ void pp_lit(nlsat::explain::imp & ex, nlsat::literal l) {
     std::cout << std::endl;
 }
 #endif
-
