@@ -98,7 +98,7 @@ struct goal2nlsat::imp {
         ptr_buffer<polynomial::polynomial> ps;
         polynomial::factors fs(m_pm);
         m_pm.factor(p, fs, m_fparams);
-        TRACE("goal2nlsat_bug", tout << "factors:\n" << fs << "\n";); 
+        TRACE(goal2nlsat_bug, tout << "factors:\n" << fs << "\n";); 
         SASSERT(fs.distinct_factors() > 0);
         for (unsigned i = 0; i < fs.distinct_factors(); i++) {
             ps.push_back(fs[i]);
@@ -106,7 +106,7 @@ struct goal2nlsat::imp {
         }
         if (m_qm.is_neg(fs.get_constant()))             
             k = flip(k);            
-        return m_solver.mk_ineq_atom(k, ps.size(), ps.c_ptr(), is_even.c_ptr());
+        return m_solver.mk_ineq_atom(k, ps.size(), ps.data(), is_even.data());
     }
 
     nlsat::literal process_atom(app * f, nlsat::atom::kind k) {
@@ -126,7 +126,7 @@ struct goal2nlsat::imp {
         m_qm.neg(d2);
         polynomial_ref p(m_pm);
         p = m_pm.addmul(d1, m_pm.mk_unit(), p1, d2, m_pm.mk_unit(), p2);
-        TRACE("goal2nlsat_bug", tout << mk_pp(f, m) << " p: " << p << "\nk: " << k << "\n";);
+        TRACE(goal2nlsat_bug, tout << mk_pp(f, m) << " p: " << p << "\nk: " << k << "\n";);
         if (is_const(p)) {
             int sign;
             if (is_zero(p))
@@ -194,7 +194,7 @@ struct goal2nlsat::imp {
                 switch (to_app(f)->get_decl_kind()) {
                 case OP_TRUE:
                 case OP_FALSE:
-                    TRACE("goal2nlsat", tout << "f: " << mk_pp(f, m) << "\n";);
+                    TRACE(goal2nlsat, tout << "f: " << mk_pp(f, m) << "\n";);
                     throw tactic_exception("apply simplify before applying nlsat");
                 case OP_AND:
                 case OP_OR:
@@ -248,11 +248,11 @@ struct goal2nlsat::imp {
         for (unsigned i = 0; i < num_lits; i++) {
             ls.push_back(process_literal(lits[i]));
         }
-        m_solver.mk_clause(ls.size(), ls.c_ptr(), dep);
+        m_solver.mk_clause(ls.size(), ls.data(), dep);
     }
 
     void operator()(goal const & g) {
-        TRACE("goal2nlsat", g.display(tout););
+        TRACE(goal2nlsat, g.display(tout););
         if (has_term_ite(g))
             throw tactic_exception("eliminate term-ite before applying nlsat");
         unsigned sz = g.size();
