@@ -173,7 +173,7 @@ namespace opt {
                 m_pb2bv(m_solver->get_assertion(i), tmp);                
                 g->assert_expr(tmp);
             }
-            TRACE("opt", g->display(tout););
+            TRACE(opt, g->display(tout););
             tactic_ref simplify = mk_nnf_tactic(m);
             proof_converter_ref pc;
             expr_dependency_ref core(m);
@@ -185,7 +185,7 @@ namespace opt {
             for (unsigned i = 0; i < r->size(); ++i) {
                 m_bvsls->assert_expr(r->form(i));
             }
-            TRACE("opt", m_bvsls->display(tout););
+            TRACE(opt, m_bvsls->display(tout););
         }
 
         void pbsls_opt(model_ref& mdl) {
@@ -212,7 +212,7 @@ namespace opt {
             m_bvsls = alloc(bvsls_opt_engine, m, m_params);            
             assertions2sls();
             expr_ref objective = soft2bv(m_soft, m_weights);
-            TRACE("opt", tout << objective << "\n";);
+            TRACE(opt, tout << objective << "\n";);
             opt_result res(m);
             res.is_sat = l_undef;
             try {
