@@ -42,7 +42,6 @@ Revision History:
 #include "sat/sat_cut_simplifier.h"
 #include "sat/sat_probing.h"
 #include "sat/sat_mus.h"
-#include "sat/sat_binspr.h"
 #include "sat/sat_drat.h"
 #include "sat/sat_parallel.h"
 #include "sat/sat_local_search.h"
@@ -90,7 +89,7 @@ namespace sat {
     
     class solver : public solver_core {
     public:
-        struct abort_solver {};
+        struct abort_solver : public std::exception {};
     protected:
         enum search_state { s_sat, s_unsat };
 
@@ -114,7 +113,6 @@ namespace sat {
         probing                 m_probing;
         bool                    m_is_probing { false };
         mus                     m_mus;           // MUS for minimal core extraction
-        binspr                  m_binspr;
         bool                    m_inconsistent;
         bool                    m_searching;
         // A conflict is usually a single justification. That is, a justification
@@ -152,6 +150,7 @@ namespace sat {
         bool_vector             m_phase; 
         bool_vector             m_best_phase;
         bool_vector             m_prev_phase;
+        bool                    m_new_best_phase = false;
         svector<char>           m_assigned_since_gc;
         search_state            m_search_state; 
         unsigned                m_search_unsat_conflicts;
@@ -214,7 +213,6 @@ namespace sat {
         friend class cleaner;
         friend class asymm_branch;
         friend class big;
-        friend class binspr;
         friend class drat;
         friend class elim_eqs;
         friend class bcd;
@@ -228,7 +226,7 @@ namespace sat {
         friend class parallel;
         friend class lookahead;
         friend class local_search;
-        friend class ddfw;
+        friend class ddfw_wrapper;
         friend class prob;
         friend class unit_walk;
         friend struct mk_stat;
@@ -380,6 +378,9 @@ namespace sat {
         bool was_eliminated(literal l) const { return was_eliminated(l.var()); }
         void set_phase(literal l) override { if (l.var() < num_vars()) m_best_phase[l.var()] = m_phase[l.var()] = !l.sign(); }
         bool get_phase(bool_var b) { return m_phase.get(b, false); }
+        bool get_best_phase(bool_var b) { return m_best_phase.get(b, false); }
+        void set_has_new_best_phase(bool b) { m_new_best_phase = b; }
+        bool has_new_best_phase() const { return m_new_best_phase; }
         void move_to_front(bool_var b);
         unsigned scope_lvl() const { return m_scope_lvl; }
         unsigned search_lvl() const { return m_search_lvl; }
@@ -394,7 +395,7 @@ namespace sat {
         unsigned trail_size() const { return m_trail.size(); }
         literal  scope_literal(unsigned n) const { return m_trail[m_scopes[n].m_trail_lim]; }
         void assign(literal l, justification j) {
-            TRACE("sat_assign", tout << l << " previous value: " << value(l) << " j: " << j << "\n";);
+            TRACE(sat_assign, tout << l << " previous value: " << value(l) << " j: " << j << "\n";);
             switch (value(l)) {
             case l_false: set_conflict(j, ~l); break;
             case l_undef: assign_core(l, j); break;
@@ -402,7 +403,7 @@ namespace sat {
             }
         }
         void update_assign(literal l, justification j) {
-            if (j.level() == 0 && !m_trim) 
+            if (j.level() == 0 && !m_trim && lvl(l) != 0) 
                 m_justification[l.var()] = j;
         }
         void assign_unit(literal l) { assign(l, justification(0)); }
@@ -417,7 +418,7 @@ namespace sat {
         bool limit_reached() {
             if (!m_rlimit.inc()) {
                 m_model_is_current = false;
-                TRACE("sat", tout << "canceled\n";);
+                TRACE(sat, tout << "canceled\n";);
                 m_reason_unknown = "sat.canceled";
                 return true;
             }
