@@ -39,7 +39,7 @@ Notes:
 #include "model/model_v2_pp.h"
 #include "model/model_evaluator.h"
 #include "sat/sat_solver.h"
-#include "sat/sat_params.hpp"
+#include "params/sat_params.hpp"
 #include "sat/smt/euf_solver.h"
 #include "sat/tactic/goal2sat.h"
 #include "sat/tactic/sat2goal.h"
@@ -137,7 +137,10 @@ public:
         for (unsigned h : m_fmls_head_lim) result->m_fmls_head_lim.push_back(h);
         for (expr* f : m_internalized_fmls) result->m_internalized_fmls.push_back(tr(f));
         if (m_mcs.back()) result->m_mcs.push_back(m_mcs.back()->translate(tr));
-        if (m_sat_mc) result->m_sat_mc = dynamic_cast<sat2goal::mc*>(m_sat_mc->translate(tr));
+        if (m_sat_mc) {
+            m_sat_mc->flush_smc(m_solver, m_map);
+            result->m_sat_mc = static_cast<sat2goal::mc*>(m_sat_mc->translate(tr));
+        }
         result->m_has_uninterpreted = m_has_uninterpreted;
         // copy m_bb_rewriter?
         result->m_internalized_converted = m_internalized_converted;
@@ -210,7 +213,7 @@ public:
             }
         }
 
-        TRACE("sat", tout << _assumptions << "\n";);
+        TRACE(sat, tout << _assumptions << "\n";);
         m_dep2asm.reset();
         lbool r = internalize_formulas();
         if (r != l_true) return r;
@@ -227,10 +230,10 @@ public:
             r = m_solver.check(m_asms.size(), m_asms.data());
         }
         catch (z3_exception& ex) {
-            IF_VERBOSE(1, verbose_stream() << "exception: " << ex.msg() << "\n";);
+            IF_VERBOSE(1, verbose_stream() << "exception: " << ex.what() << "\n";);
             if (m.inc()) {
                 reason_set = true;
-                set_reason_unknown(std::string("(sat.giveup ") + ex.msg() + ')');
+                set_reason_unknown(std::string("(sat.giveup ") + ex.what() + ')');
             }
             r = l_undef;            
         }
@@ -363,7 +366,7 @@ public:
 
     ast_manager& get_manager() const override { return m; }
     void assert_expr_core(expr * t) override {
-        TRACE("goal2sat", tout << mk_pp(t, m) << "\n";);
+        TRACE(goal2sat, tout << mk_pp(t, m) << "\n";);
         m_is_cnf &= is_clause(t);
         m_fmls.push_back(t);
     }
@@ -480,11 +483,12 @@ public:
 
     expr* congruence_next(expr* e) override { return e; }
     expr* congruence_root(expr* e) override { return e; }
+    expr_ref congruence_explain(expr* a, expr* b) override { return expr_ref(m.mk_eq(a, b), m); }
 
     
     lbool get_consequences_core(expr_ref_vector const& assumptions, expr_ref_vector const& vars, expr_ref_vector& conseq) override {
         init_preprocess();
-        TRACE("sat", tout << assumptions << "\n" << vars << "\n";);
+        TRACE(sat, tout << assumptions << "\n" << vars << "\n";);
         sat::literal_vector asms;
         sat::bool_var_vector bvars;
         vector<sat::literal_vector> lconseq;
@@ -509,7 +513,7 @@ public:
         // the consequences that cover them.
         u_map<unsigned> bool_var2conseq;
         for (unsigned i = 0; i < lconseq.size(); ++i) {
-            TRACE("sat", tout << lconseq[i] << "\n";);
+            TRACE(sat, tout << lconseq[i] << "\n";);
             bool_var2conseq.insert(lconseq[i][0].var(), i);
         }
 
@@ -603,7 +607,7 @@ public:
             m_cached_mc = m_mcs.back();
             m_cached_mc = concat(solver::get_model_converter().get(), m_cached_mc.get());
             m_cached_mc = concat(m_cached_mc.get(), m_sat_mc.get());
-            TRACE("sat", m_cached_mc->display(tout););
+            TRACE(sat, m_cached_mc->display(tout););
             return m_cached_mc;
         }
         else {
@@ -623,7 +627,7 @@ public:
         s2g(m_solver, m_map, m_params, g, m_sat_mc);
         m_internalized_fmls.reset();
         g.get_formulas(m_internalized_fmls);
-        TRACE("sat", m_solver.display(tout); tout << m_internalized_fmls << "\n";);
+        TRACE(sat, m_solver.display(tout); tout << m_internalized_fmls << "\n";);
         m_internalized_converted = true;
     }
 
@@ -734,7 +738,7 @@ private:
             m_has_uninterpreted = true;
             std::stringstream strm;
             strm << "(sat.giveup interpreted functions sent to SAT solver " << funs <<")";
-            TRACE("sat", tout << strm.str() << "\n";);
+            TRACE(sat, tout << strm.str() << "\n";);
             IF_VERBOSE(1, verbose_stream() << strm.str() << "\n";);
             set_reason_unknown(strm.str());
             return l_undef;
@@ -767,7 +771,7 @@ private:
             throw default_exception("generation of proof objects is not supported in this mode");
         }
         SASSERT(!g->proofs_enabled());
-        TRACE("sat", m_solver.display(tout); g->display(tout););
+        TRACE(sat, m_solver.display(tout); g->display(tout););
 
         try {
             if (m_is_cnf) {
@@ -779,9 +783,9 @@ private:
             }
         }
         catch (tactic_exception & ex) {
-            IF_VERBOSE(1, verbose_stream() << "exception in tactic " << ex.msg() << "\n";);
-            set_reason_unknown(ex.msg());
-            TRACE("sat", tout << "exception: " << ex.msg() << "\n";);
+            IF_VERBOSE(1, verbose_stream() << "exception in tactic " << ex.what() << "\n";);
+            set_reason_unknown(ex.what());
+            TRACE(sat, tout << "exception: " << ex.what() << "\n";);
             m_preprocess = nullptr;
             m_bb_rewriter = nullptr;
             return l_undef;
@@ -799,7 +803,7 @@ private:
         g = m_subgoals[0];
         m_pc = g->pc();
         m_mcs.set(m_mcs.size()-1, concat(m_mcs.back(), g->mc()));
-        TRACE("sat", g->display_with_dependencies(tout););
+        TRACE(sat, g->display_with_dependencies(tout););
 
         // ensure that if goal is already internalized, then import mc from m_solver.
 
@@ -877,12 +881,12 @@ private:
                     bvars.push_back(b);
                 }
             }
-            CTRACE("sat", internalized, tout << "var: " << bvars << "\n";);
+            CTRACE(sat, internalized, tout << "var: " << bvars << "\n";);
         }
         else if (is_uninterp_const(v) && bvutil.is_bv(v)) {
             // variable does not occur in assertions, so is unconstrained.
         }
-        CTRACE("sat", !internalized, tout << "unhandled variable " << mk_pp(v, m) << "\n";);
+        CTRACE(sat, !internalized, tout << "unhandled variable " << mk_pp(v, m) << "\n";);
         return internalized;
     }
 
@@ -903,7 +907,7 @@ private:
                 }
             }
             else {
-                TRACE("sat", tout << "variable is not bound " << mk_pp(v, m) << "\n";);
+                TRACE(sat, tout << "variable is not bound " << mk_pp(v, m) << "\n";);
                 return false;
             }
         }
@@ -1017,7 +1021,7 @@ private:
                 }
             }
         }
-        CTRACE("sat", m_dep2asm.size() != m_asms.size(), 
+        CTRACE(sat, m_dep2asm.size() != m_asms.size(), 
                tout << m_dep2asm.size() << " vs " << m_asms.size() << "\n";
                tout << m_asms << "\n";
                for (auto const& kv : m_dep2asm) {
@@ -1036,7 +1040,7 @@ private:
         u_map<expr*> asm2dep;
         extract_asm2dep(asm2dep);
         sat::literal_vector const& core = m_solver.get_core();
-        TRACE("sat",
+        TRACE(sat,
               for (auto const& kv : m_dep2asm) {
                   tout << mk_pp(kv.m_key, m) << " |-> " << sat::literal(kv.m_value) << "\n";
               }
@@ -1074,7 +1078,7 @@ private:
     }
 
     void get_model_core(model_ref & mdl) override {
-        TRACE("sat", tout << "retrieve model " << (m_solver.model_is_current()?"present":"absent") << "\n";);
+        TRACE(sat, tout << "retrieve model " << (m_solver.model_is_current()?"present":"absent") << "\n";);
         if (!m_solver.model_is_current()) {
             mdl = nullptr;
             return;
@@ -1083,13 +1087,13 @@ private:
             mdl = nullptr;
             return;
         }
-        TRACE("sat", m_solver.display_model(tout););
-        CTRACE("sat", m_sat_mc, m_sat_mc->display(tout););
+        TRACE(sat, m_solver.display_model(tout););
+        CTRACE(sat, m_sat_mc, m_sat_mc->display(tout););
         sat::model ll_m = m_solver.get_model();
         mdl = alloc(model, m);
-        if (m_sat_mc) {
+        if (m_sat_mc) 
             (*m_sat_mc)(ll_m);
-        }        
+        
         expr_ref_vector var2expr(m);
         m_map.mk_var_inv(var2expr);
         
@@ -1110,16 +1114,18 @@ private:
             }
         }
 
-        TRACE("sat", m_solver.display(tout););
+        TRACE(sat, m_solver.display(tout););
+           
         if (m_sat_mc) {
             (*m_sat_mc)(mdl);
         }
         m_goal2sat.update_model(mdl);
         if (m_mcs.back()) {      
-            TRACE("sat", m_mcs.back()->display(tout););
+            TRACE(sat, m_mcs.back()->display(tout););
             (*m_mcs.back())(mdl);
         }
-        TRACE("sat", model_smt2_pp(tout, m, *mdl, 0););        
+
+        TRACE(sat, model_smt2_pp(tout, m, *mdl, 0););        
 
         if (!gparams::get_ref().get_bool("model_validate", false)) {
             return;
@@ -1135,7 +1141,7 @@ private:
             eval(f, tmp);
             if (m.limit().is_canceled())
                 return;
-            CTRACE("sat", !m.is_true(tmp),
+            CTRACE(sat, !m.is_true(tmp),
                    tout << "Evaluation failed: " << mk_pp(f, m) << " to " << tmp << "\n";
                    model_smt2_pp(tout, m, *(mdl.get()), 0););
             if (m.is_false(tmp)) {
