@@ -23,7 +23,7 @@ namespace array {
     sat::literal solver::internalize(expr* e, bool sign, bool root) { 
         SASSERT(m.is_bool(e));
         if (!visit_rec(m, e, sign, root)) {
-            TRACE("array", tout << mk_pp(e, m) << "\n";);
+            TRACE(array, tout << mk_pp(e, m) << "\n";);
             return sat::null_literal;
         }
         auto lit = expr2literal(e);
