@@ -21,7 +21,10 @@ Author:
 namespace bv {
 
     bool solver::check_delay_internalized(expr* e) {
-        if (!ctx.is_relevant(e))
+        euf::enode* n = expr2enode(e);
+        if (!n)
+            return true;
+        if (!ctx.is_relevant(n))
             return true;
         if (get_internalize_mode(e) != internalize_mode::delay_i)
             return true;
@@ -44,7 +47,7 @@ namespace bv {
             return true;
         unsigned num_vars = e->get_num_args();
         for (expr* arg : *e) 
-            if (!m.is_value(arg))
+            if (m.is_value(arg))
                 --num_vars;
         if (num_vars <= 1) 
             return true;
@@ -69,16 +72,67 @@ namespace bv {
         return expr_ref(bv.mk_numeral(val, get_bv_size(v)), m);
     }
 
+    /**
+       \brief expose the multiplication circuit lazily.
+       It adds clauses for multiplier output one by one to enforce
+       the semantics of multipliers.
+     */
+
+    bool solver::check_lazy_mul(app* e, expr* arg_value, expr* mul_value) {
+        SASSERT(e->get_num_args() >= 2);
+        expr_ref_vector args(m), new_args(m), new_out(m);
+        lazy_mul* lz = nullptr;
+        rational v0, v1;
+        unsigned sz, diff = 0;
+        VERIFY(bv.is_numeral(arg_value, v0, sz));
+        VERIFY(bv.is_numeral(mul_value, v1));
+        for (diff = 0; diff < sz; ++diff) 
+            if (v0.get_bit(diff) != v1.get_bit(diff))
+                break;
+        SASSERT(diff < sz);
+        auto set_bits = [&](unsigned j, expr_ref_vector& bits) {
+            bits.reset();
+            for (unsigned i = 0; i < sz; ++i)
+                bits.push_back(bv.mk_bit2bool(e->get_arg(0), j));            
+        };
+        if (!m_lazymul.find(e, lz)) {
+            set_bits(0, args);
+            for (unsigned j = 1; j < e->get_num_args(); ++j) {
+                new_out.reset();
+                set_bits(j, new_args);
+                m_bb.mk_multiplier(sz, args.data(), new_args.data(), new_out);
+                new_out.swap(args);
+            }
+            lz = alloc(lazy_mul, e, args);
+            m_lazymul.insert(e, lz);
+            ctx.push(new_obj_trail(lz));
+            ctx.push(insert_obj_map(m_lazymul, e));
+        }
+        if (lz->m_out.size() == lz->m_bits)
+            return false;
+        for (unsigned i = lz->m_bits; i <= diff; ++i) {
+            sat::literal bit1 = mk_literal(lz->m_out.get(i));
+            sat::literal bit2 = mk_literal(bv.mk_bit2bool(e, i));
+            add_equiv(bit1, bit2);
+        }
+        ctx.push(value_trail(lz->m_bits));
+        IF_VERBOSE(1, verbose_stream() << "expand lazy mul " << mk_pp(e, m) << " to " << diff << "\n");
+        lz->m_bits = diff;
+        return false;
+    }
+
     bool solver::check_mul(app* e) {
         SASSERT(e->get_num_args() >= 2);
         expr_ref_vector args(m);
         euf::enode* n = expr2enode(e);
+        if (!reflect())
+            return false;
         auto r1 = eval_bv(n);
         auto r2 = eval_args(n, args);
         if (r1 == r2)
             return true;
 
-        TRACE("bv", tout << mk_bounded_pp(e, m) << " evaluates to " << r1 << " arguments: " << args << "\n";);
+        TRACE(bv, tout << mk_bounded_pp(e, m) << " evaluates to " << r1 << " arguments: " << args << "\n";);
         // check x*0 = 0
         if (!check_mul_zero(e, args, r1, r2))
             return false;
@@ -91,6 +145,12 @@ namespace bv {
         if (!check_mul_invertibility(e, args, r1))
             return false;
 
+#if 0
+        // unsound?
+
+        if (!check_lazy_mul(e, r1, r2))
+            return false;
+#endif
         // Some other possible approaches:
         // algebraic rules:
         // x*(y+z), and there are nodes for x*y or x*z -> x*(y+z) = x*y + x*z
@@ -147,7 +207,7 @@ namespace bv {
         };
         auto add_inv = [&](expr* s) {
             inv = invert(s, n);
-            TRACE("bv", tout << "enforce " << inv << "\n";);
+            TRACE(bv, tout << "enforce " << inv << "\n";);
             add_unit(eq_internalize(inv, n));
         };
         bool ok = true;
@@ -172,18 +232,17 @@ namespace bv {
     bool solver::check_mul_zero(app* n, expr_ref_vector const& arg_values, expr* mul_value, expr* arg_value) {
         SASSERT(mul_value != arg_value);
         SASSERT(!(bv.is_zero(mul_value) && bv.is_zero(arg_value)));
-        if (bv.is_zero(arg_value)) {
+        if (bv.is_zero(arg_value) && false) {
             unsigned sz = n->get_num_args();
             expr_ref_vector args(m, sz, n->get_args());
             for (unsigned i = 0; i < sz && !s().inconsistent(); ++i) {
-
                 args[i] = arg_value;
                 expr_ref r(m.mk_app(n->get_decl(), args), m);
                 set_delay_internalize(r, internalize_mode::init_bits_only_i); // do not bit-blast this multiplier.
                 args[i] = n->get_arg(i);                
                 add_unit(eq_internalize(r, arg_value));
             }
-            IF_VERBOSE(2, verbose_stream() << "delay internalize @" << s().scope_lvl() << "\n");
+            IF_VERBOSE(2, verbose_stream() << "delay internalize @" << s().scope_lvl() << " " << mk_pp(n, m) << "\n");
             return false;
         }
         if (bv.is_zero(mul_value)) {
@@ -221,14 +280,14 @@ namespace bv {
             expr_ref mul1(m.mk_app(n->get_decl(), arg_values[0], n->get_arg(1)), m);
             set_delay_internalize(mul1, internalize_mode::init_bits_only_i);
             add_unit(eq_internalize(mul1, n->get_arg(1)));
-            TRACE("bv", tout << mul1 << "\n";);
+            TRACE(bv, tout << mul1 << "\n";);
             return false;
         }
         if (bv.is_one(arg_values[1])) {
             expr_ref mul1(m.mk_app(n->get_decl(), n->get_arg(0), arg_values[1]), m);
             set_delay_internalize(mul1, internalize_mode::init_bits_only_i);
             add_unit(eq_internalize(mul1, n->get_arg(0)));
-            TRACE("bv", tout << mul1 << "\n";);
+            TRACE(bv, tout << mul1 << "\n";);
             return false;
         }
         return true;
@@ -245,7 +304,7 @@ namespace bv {
             return;
         expr_ref tmp = literal2expr(bits.back());
         for (unsigned i = bits.size() - 1; i-- > 0; ) {
-            auto b = bits[i];
+            sat::literal b = bits[i];
             tmp = m.mk_or(literal2expr(b), tmp);
             xs.push_back(tmp);
         }
@@ -356,6 +415,8 @@ namespace bv {
             return internalize_mode::no_delay_i;
         if (!get_config().m_bv_delay)
             return internalize_mode::no_delay_i;
+        if (!reflect())
+            return internalize_mode::no_delay_i;
         internalize_mode mode;
         switch (to_app(e)->get_decl_kind()) {
         case OP_BMUL: 
