@@ -16,8 +16,7 @@ Author:
 Revision History:
 
 --*/
-#ifndef SMT_BOOL_VAR_DATA_H_
-#define SMT_BOOL_VAR_DATA_H_
+#pragma once
 
 #include "smt/smt_b_justification.h"
 
@@ -134,5 +133,4 @@ namespace smt {
     };
 };
 
-#endif /* SMT_BOOL_VAR_DATA_H_ */
 
