@@ -19,6 +19,7 @@ Revision History:
 #pragma once
 
 #include "ast/quantifier_stat.h"
+#include "ast/simplifiers/dependent_expr_state.h"
 #include "smt/smt_clause.h"
 #include "smt/smt_setup.h"
 #include "smt/smt_enode.h"
@@ -132,6 +133,11 @@ namespace smt {
         bool                        m_internalizing_assertions = false;
         lbool                       m_internal_completed = l_undef;
 
+        scoped_ptr<dependent_expr_simplifier> m_simplifier;
+        scoped_ptr<base_dependent_expr_state> m_fmls;
+
+        svector<double> m_lit_scores[2];
+
 
         // -----------------------------------
         //
@@ -263,6 +269,7 @@ namespace smt {
         // ----------------------------------
         vector<std::pair<expr_ref, expr_ref>> m_values;
         void initialize_value(expr* var, expr* value);
+        void initialize_values();
 
 
         // -----------------------------------
@@ -1292,6 +1299,8 @@ namespace smt {
 
         virtual bool resolve_conflict();
 
+        void add_scores(unsigned n, literal const *lits);
+
 
         // -----------------------------------
         //
