@@ -34,7 +34,7 @@ namespace smt {
     }
 
     std::ostream& operator<<(std::ostream& out, model_value_dependency const& src) {
-        if (src.is_fresh_value()) return out << "fresh!" << src.get_value()->get_idx();
+        if (src.is_fresh_value()) return out << "fresh!" << src.get_value()->get_idx() << "#" << src.get_value()->get_src()->get_id();
         else return out << "#" << src.get_enode()->get_owner_id();
     }
 
@@ -63,7 +63,7 @@ namespace smt {
         // PARAM-TODO smt_params ---> params_ref
         m_model = alloc(proto_model, m); // , m_context->get_fparams());
         for (theory* th : m_context->theories()) {
-            TRACE("model_generator_bug", tout << "init_model for theory: " << th->get_name() << "\n";);
+            TRACE(model_generator_bug, tout << "init_model for theory: " << th->get_name() << "\n";);
             th->init_model(*this);
         }
     }
@@ -97,7 +97,7 @@ namespace smt {
                 sort * s      = r->get_sort();
                 model_value_proc * proc = nullptr;
                 if (m.is_bool(s)) {
-                    CTRACE("model", m_context->get_assignment(r) == l_undef, 
+                    CTRACE(model, m_context->get_assignment(r) == l_undef, 
                            tout << mk_pp(r->get_expr(), m) << "\n";);
                     if (m_context->get_assignment(r) == l_true)
                         proc = alloc(expr_wrapper_proc, m.mk_true());
@@ -115,8 +115,11 @@ namespace smt {
                             SASSERT(proc);
                         }
                         else {
-                            TRACE("model", tout << "creating fresh value for #" << mk_pp(r->get_expr(), m) << "\n";);
-                            proc = alloc(fresh_value_proc, mk_extra_fresh_value(r->get_sort()));
+                            TRACE(model, tout << "creating fresh value for #" 
+                                << r->get_expr_id() << " " 
+                                << mk_bounded_pp(r->get_expr(), m) << " " 
+                                << mk_pp(r->get_sort(), m) << "\n";);
+                            proc = alloc(fresh_value_proc, mk_extra_fresh_value(r->get_expr(), r->get_sort()));
                         }
                     }
                     else {
@@ -137,7 +140,7 @@ namespace smt {
         if (!m.is_model_value(n)) {
             sort * s = r->get_sort();
             n = m_model->get_fresh_value(s);
-            CTRACE("model", n == 0, 
+            CTRACE(model, n == 0, 
                    tout << mk_pp(r->get_expr(), m) << "\nsort:\n" << mk_pp(s, m) << "\n";
                    tout << "is_finite: " << m_model->is_finite(s) << "\n";);
         }
@@ -181,15 +184,20 @@ namespace smt {
             if (already_traversed.contains(s))
                 return true;
             bool visited = true;
+            TRACE(mg_top_sort, tout << "fresh value of sort " << mk_pp(s, m) << "\n";
+            for (enode* r : roots)
+                if (r->get_sort() == s)
+                    tout << mk_pp(r->get_expr(), m) << "\n";
+                );
             for (enode * r : roots) {
                 if (r->get_sort() != s)
                     continue;
                 SASSERT(r == r->get_root());
                 if (root2proc[r]->is_fresh()) 
                     continue; // r is associated with a fresh value...
-                TRACE("mg_top_sort", tout << "fresh!" << src.get_value()->get_idx() << " -> #" << r->get_owner_id() << " " << mk_pp(r->get_sort(), m) << "\n";);
+                TRACE(mg_top_sort, tout << "fresh!" << src.get_value()->get_idx() << " -> #" << r->get_owner_id() << " " << mk_pp(r->get_sort(), m) << "\n";);
                 visit_child(source(r), colors, todo, visited);
-                TRACE("mg_top_sort", tout << "visited: " << visited << ", todo.size(): " << todo.size() << "\n";);
+                TRACE(mg_top_sort, tout << "visited: " << visited << ", todo.size(): " << todo.size() << "\n";);
             }
             already_traversed.insert(s);
             return visited;
@@ -206,7 +214,7 @@ namespace smt {
         for (model_value_dependency const& dep : dependencies) {
             visit_child(dep, colors, todo, visited);
         }
-        TRACE("mg_top_sort",
+        TRACE(mg_top_sort,
               tout << "src: " << src << " ";
               tout << mk_pp(n->get_expr(), m) << "\n";
               for (model_value_dependency const& dep : dependencies) {
@@ -223,7 +231,7 @@ namespace smt {
                                          obj_hashtable<sort> & already_traversed, 
                                          svector<source> & todo,
                                          svector<source> & sorted_sources) {
-        TRACE("mg_top_sort", tout << "process source, is_fresh: " << src.is_fresh_value() << " ";
+        TRACE(mg_top_sort, tout << "process source, is_fresh: " << src.is_fresh_value() << " ";
               tout << src << ", todo.size(): " << todo.size() << "\n";);
         int color     = get_color(colors, src);
         SASSERT(color != Grey);
@@ -233,7 +241,7 @@ namespace smt {
         todo.push_back(src);
         while (!todo.empty()) {
             source curr = todo.back();
-            TRACE("mg_top_sort", tout << "current source, is_fresh: " << curr.is_fresh_value() << " ";
+            TRACE(mg_top_sort, tout << "current source, is_fresh: " << curr.is_fresh_value() << " ";
                   tout << curr << ", todo.size(): " << todo.size() << "\n";);
             switch (get_color(colors, curr)) {
             case White:
@@ -243,7 +251,7 @@ namespace smt {
             case Grey:
                 // SASSERT(visit_children(curr, roots, root2proc, colors, already_traversed, todo));
                 set_color(colors, curr, Black);
-                TRACE("mg_top_sort", tout << "append " << curr << "\n";);
+                TRACE(mg_top_sort, tout << "append " << curr << "\n";);
                 sorted_sources.push_back(curr);
                 break;
             case Black:
@@ -253,7 +261,7 @@ namespace smt {
                 UNREACHABLE();
             }
         }
-        TRACE("mg_top_sort", tout << "END process_source, todo.size(): " << todo.size() << "\n";);
+        TRACE(mg_top_sort, tout << "END process_source, todo.size(): " << todo.size() << "\n";);
     }
 
     /**
@@ -298,37 +306,44 @@ namespace smt {
         expr_ref_vector dependency_values(m);
         mk_value_procs(root2proc, roots, procs);
         top_sort_sources(roots, root2proc, sources);
-        TRACE("sorted_sources",
+        TRACE(sorted_sources,
               for (source const& curr : sources) {
                   if (curr.is_fresh_value()) {
                       tout << curr << " " << mk_pp(curr.get_value()->get_sort(), m) << "\n";
                   }
                   else {
                       enode * n = curr.get_enode();
+                      sort* s = n->get_sort();
                       SASSERT(n->get_root() == n);
-                      tout << mk_pp(n->get_expr(), m) << "\n";
-                      sort * s = n->get_sort();
                       tout << curr << " " << mk_pp(s, m);
-                      tout << " is_fresh: " << root2proc[n]->is_fresh() << "\n";
+                      tout << mk_bounded_pp(n->get_expr(), m) << " ";
+                      tout << " is_fresh: " << root2proc[n]->is_fresh() << " - deps: ";
+                      dependencies.reset();
+                      model_value_proc* proc = root2proc[n];
+                      SASSERT(proc);
+                      proc->get_dependencies(dependencies);
+                      for (auto const& d : dependencies)
+                          tout << d << " ";
+                      tout << "\n";
                   }
               }
-              m_context->display(tout);
+              // m_context->display(tout);
               );
 
 
         for (source const& curr : sources) {
             if (curr.is_fresh_value()) {
                 sort * s = curr.get_value()->get_sort();
-                TRACE("model_fresh_bug", tout << curr << " : " << mk_pp(s, m) << " " << curr.get_value()->get_value() << "\n";);
+                TRACE(model_fresh_bug, tout << curr << " : " << mk_pp(s, m) << " " << curr.get_value()->get_value() << "\n";);
                 expr * val = m_model->get_fresh_value(s);
-                TRACE("model_fresh_bug", tout << curr << " := #" << (val == nullptr ? UINT_MAX : val->get_id()) << "\n";);
+                TRACE(model_fresh_bug, tout << curr << " := #" << (val == nullptr ? UINT_MAX : val->get_id()) << "\n";);
                 m_asts.push_back(val);
                 curr.get_value()->set_value(val);
             }
             else {
                 enode * n = curr.get_enode();
                 SASSERT(n->get_root() == n);
-                TRACE("mg_top_sort", tout << curr << "\n";);
+                TRACE(mg_top_sort, tout << curr << "\n";);
                 app* val = nullptr;
                 if (m.is_value(n->get_expr()))
                     val = to_app(n->get_expr());
@@ -340,14 +355,20 @@ namespace smt {
                     proc->get_dependencies(dependencies);
                     for (model_value_dependency const& d : dependencies) {
                         if (d.is_fresh_value()) {
-                            CTRACE("mg_top_sort", !d.get_value()->get_value(), 
-                                   tout << "#" << n->get_owner_id() << " " << mk_pp(n->get_expr(), m) << " -> " << d << "\n";);
-                            SASSERT(d.get_value()->get_value());
-                            dependency_values.push_back(d.get_value()->get_value());
+                            expr* val = d.get_value()->get_value();
+                            CTRACE(mg_top_sort, !val,
+                                tout << "#" << n->get_owner_id() << " " <<
+                                mk_pp(n->get_expr(), m) << " -> " << d << "\n";);
+                            // there is a cyclic dependency for default(A), where A
+                            // is an array of a datatype with the datatype using A.
+                            if (!val)
+                                val = m_model->get_some_value(d.get_value()->get_sort());
+                            SASSERT(val);
+                            dependency_values.push_back(val);
                         }
                         else {
                             enode * child = d.get_enode();
-                            TRACE("mg_top_sort", tout << "#" << n->get_owner_id() << " (" << mk_pp(n->get_expr(), m) << "): " 
+                            TRACE(mg_top_sort, tout << "#" << n->get_owner_id() << " (" << mk_pp(n->get_expr(), m) << "): " 
                                   << mk_pp(child->get_expr(), m) << " " << mk_pp(child->get_root()->get_expr(), m) << "\n";);
                             child = child->get_root();
                             dependency_values.push_back(m_root2value[child]);
@@ -364,7 +385,7 @@ namespace smt {
         for (enode * n : m_context->enodes()) {
             if (is_uninterp_const(n->get_expr()) && m_context->is_relevant(n)) {
                 func_decl * d = n->get_expr()->get_decl();
-                TRACE("mg_top_sort", tout << d->get_name() << " " << (m_hidden_ufs.contains(d)?"hidden":"visible") << "\n";);
+                TRACE(mg_top_sort, tout << d->get_name() << " " << (m_hidden_ufs.contains(d)?"hidden":"visible") << "\n";);
                 if (m_hidden_ufs.contains(d)) continue;
                 expr * val    = get_value(n);
                 m_model->register_decl(d, val);
@@ -429,7 +450,7 @@ namespace smt {
                 SASSERT(m_model->has_interpretation(f));
                 SASSERT(m_model->get_func_interp(f) == fi);
                 // The entry must be new because n->get_cg() == n
-                TRACE("model", 
+                TRACE(model, 
                       tout << "insert new entry for:\n" << mk_ismt2_pp(n->get_expr(), m) << "\nargs: ";
                       for (unsigned i = 0; i < num_args; i++) {
                           tout << "#" << n->get_arg(i)->get_owner_id() << " ";
@@ -446,8 +467,8 @@ namespace smt {
         }
     }
 
-    extra_fresh_value * model_generator::mk_extra_fresh_value(sort * s) {        
-        extra_fresh_value * r = alloc(extra_fresh_value, s, m_fresh_idx);
+    extra_fresh_value * model_generator::mk_extra_fresh_value(expr * src, sort* s) {        
+        extra_fresh_value * r = alloc(extra_fresh_value, src, s, m_fresh_idx);
         m_fresh_idx++;
         m_extra_fresh_values.push_back(r);
         return r;
@@ -485,20 +506,20 @@ namespace smt {
 
     void model_generator::register_macros() {
         unsigned num = m_context->get_num_macros();
-        TRACE("model", tout << "num. macros: " << num << "\n";);
+        TRACE(model, tout << "num. macros: " << num << "\n";);
         expr_ref v(m);
         for (unsigned i = 0; i < num; i++) {
             func_decl * f    = m_context->get_macro_interpretation(i, v);
             func_interp * fi = alloc(func_interp, m, f->get_arity());
             fi->set_else(v);
-            TRACE("model", tout << f->get_name() << "\n" << mk_pp(v, m) << "\n";);
+            TRACE(model, tout << f->get_name() << "\n" << mk_pp(v, m) << "\n";);
             m_model->register_decl(f, fi);
         }
     }
 
     proto_model * model_generator::mk_model() {
         SASSERT(!m_model);
-        TRACE("model_verbose", m_context->display(tout););
+        TRACE(model_verbose, m_context->display(tout););
         init_model();
         register_existing_model_values();
         mk_bool_model();
@@ -506,7 +527,7 @@ namespace smt {
         mk_func_interps();
         finalize_theory_models();
         register_macros();
-        TRACE("model", model_v2_pp(tout, *m_model, true););        
+        TRACE(model, model_v2_pp(tout, *m_model, true););        
         return m_model.get();
     }
     
