@@ -1,23 +1,34 @@
  /*++
 Copyright (c) 2011 Microsoft Corporation
  
- Module Name:
+Module Name:
  
     bit_blaster_tactic.h
  
- Abstract:
+Author:
  
-    Apply bit-blasting to a given goal.
- 
- Author:
- 
-    Leonardo (leonardo) 2011-10-25
- 
- Notes:
+   Leonardo (leonardo) 2011-10-25
  
+Tactic Documentation:
+
+## Tactic bit-blast
+
+### Short Description
+
+Apply bit-blasting to a given goal.
+
+### Example
+
+```z3
+(declare-const x (_ BitVec 8))
+(declare-const y (_ BitVec 8))
+(assert (bvule x y))
+(apply bit-blast)
+```
+
  --*/
-#ifndef BIT_BLASTER_TACTIC_H_
-#define BIT_BLASTER_TACTIC_H_
+
+#pragma once
  
 #include "util/params.h"
 #include "ast/rewriter/bit_blaster/bit_blaster_rewriter.h"
@@ -29,5 +40,4 @@ tactic * mk_bit_blaster_tactic(ast_manager & m, bit_blaster_rewriter* rw, params
  /*
   ADD_TACTIC("bit-blast", "reduce bit-vector expressions into SAT.", "mk_bit_blaster_tactic(m, p)")
  */
- #endif
 
