@@ -16,8 +16,7 @@ Author:
 Notes:
    
 --*/
-#ifndef FD_SOLVER_H_
-#define FD_SOLVER_H_
+#pragma once
 
 #include "ast/ast.h"
 #include "util/params.h"
@@ -34,4 +33,3 @@ tactic * mk_parallel_qffd_tactic(ast_manager& m, params_ref const& p);
     ADD_TACTIC("pqffd", "builtin strategy for solving QF_FD problems in parallel.", "mk_parallel_qffd_tactic(m, p)")
 */
 
-#endif
