@@ -117,13 +117,13 @@ namespace lp {
 
         void fill_simple_elem(lisp_elem & lm) {
             int separator = first_separator();
-            lp_assert(-1 != separator && separator != 0);
+            SASSERT(-1 != separator && separator != 0);
             lm.m_head = m_line.substr(0, separator);
             m_line = m_line.substr(separator);
         }
 
         void fill_nested_elem(lisp_elem & lm) {
-            lp_assert(m_line[0] == '(');
+            SASSERT(m_line[0] == '(');
             m_line = m_line.substr(1);
             int separator = first_separator();
             lm.m_head = m_line.substr(0, separator);
@@ -190,11 +190,11 @@ namespace lp {
         }
 
         void adjust_right_side(formula_constraint & /* c*/, lisp_elem & /*el*/) {
-            // lp_assert(el.m_head == "0"); // do nothing for the time being
+            // SASSERT(el.m_head == "0"); // do nothing for the time being
         }
 
         void set_constraint_coeffs(formula_constraint & c, lisp_elem & el) {
-            lp_assert(el.m_elems.size() == 2);
+            SASSERT(el.m_elems.size() == 2);
             set_constraint_coeffs_on_coeff_element(c, el.m_elems[0]);
             adjust_right_side(c, el.m_elems[1]);
         }
@@ -210,7 +210,7 @@ namespace lp {
                 add_mult_elem(c, el.m_elems);
             } else if (el.m_head == "~") {
                 lisp_elem & minel = el.m_elems[0];
-                lp_assert(minel.is_simple());
+                SASSERT(minel.is_simple());
                 c.m_right_side += mpq(str_to_int(minel.m_head));
             } else {
                 std::cout << "unexpected input " << el.m_head << std::endl;
@@ -220,14 +220,14 @@ namespace lp {
         }
 
         std::string get_name(lisp_elem & name) {
-            lp_assert(name.is_simple());
-            lp_assert(!is_integer(name.m_head));
+            SASSERT(name.is_simple());
+            SASSERT(!is_integer(name.m_head));
             return name.m_head;
         }
 
 
         void add_mult_elem(formula_constraint & c, std::vector<lisp_elem> & els) {
-            lp_assert(els.size() == 2);
+            SASSERT(els.size() == 2);
             mpq coeff = get_coeff(els[0]);
             std::string col_name = get_name(els[1]);
             c.add_pair(coeff, col_name);
@@ -237,16 +237,16 @@ namespace lp {
             if (le.is_simple()) {
                 return mpq(str_to_int(le.m_head));
             } else {
-                lp_assert(le.m_head == "~");
-                lp_assert(le.size() == 1);
+                SASSERT(le.m_head == "~");
+                SASSERT(le.size() == 1);
                 lisp_elem & el = le.m_elems[0];
-                lp_assert(el.is_simple());
+                SASSERT(el.is_simple());
                 return -mpq(str_to_int(el.m_head));
             }
         }
 
         int str_to_int(std::string & s) {
-            lp_assert(is_integer(s));
+            SASSERT(is_integer(s));
             return atoi(s.c_str());
         }
 
@@ -254,7 +254,7 @@ namespace lp {
             if (el.size()) {
                 add_complex_sum_elem(c, el);
             } else {
-                lp_assert(is_integer(el.m_head));
+                SASSERT(is_integer(el.m_head));
                 int v = atoi(el.m_head.c_str());
                 mpq vr(v);
                 c.m_right_side -= vr;
