@@ -25,7 +25,6 @@
             std::cout << "    " << #MODULE << "\n";          \
         for (int i = 0; i < argc; i++)                       \
             if (test_all || strcmp(argv[i], #MODULE) == 0) { \
-                enable_trace(#MODULE);                       \
                 enable_debug(#MODULE);                       \
                 timeit timeit(true, s.c_str());              \
                 tst_##MODULE();                              \
@@ -134,7 +133,7 @@ void parse_cmd_line_args(int argc, char ** argv, bool& do_display_usage, bool& t
                 gparams::set(key, value);
             }
             catch (z3_exception& ex) {
-                std::cerr << ex.msg() << "\n";
+                std::cerr << ex.what() << "\n";
             }
         }
         i++;
@@ -176,6 +175,11 @@ int main(int argc, char ** argv) {
     TST(var_subst);
     TST(simple_parser);
     TST(api);
+    TST(api_algebraic);
+    TST(api_polynomial);
+    TST(api_pb);
+    TST(api_datalog);
+    TST(parametric_datatype);
     TST(cube_clause);
     TST(old_interval);
     TST(get_implied_equalities);
@@ -200,7 +204,9 @@ int main(int argc, char ** argv) {
     TST(egraph);
     TST(ex);
     TST(nlarith_util);
+    TST(api_ast_map);
     TST(api_bug);
+    TST(api_special_relations);
     TST(arith_rewriter);
     TST(check_assumptions);
     TST(smt_context);
@@ -211,8 +217,13 @@ int main(int argc, char ** argv) {
     TST(smt2print_parse);
     TST(substitution);
     TST(polynomial);
+    TST(polynomial_factorization);
     TST(upolynomial);
     TST(algebraic);
+    TST(algebraic_numbers);
+    TST(monomial_bounds);
+    TST(nla_intervals);
+    TST(horner);
     TST(prime_generator);
     TST(permutation);
     TST(nlsat);
@@ -270,4 +281,6 @@ int main(int argc, char ** argv) {
     TST(euf_arith_plugin);
     TST(sls_test);
     TST(scoped_vector);
+    TST(sls_seq_plugin);
+    TST(ho_matcher);
 }
