@@ -12,6 +12,7 @@ Abstract:
 Author:
 
     Leonardo de Moura (leonardo) 2006-09-11.
+    Chuyue Sun (liviasun) 2024-07-18.
 
 Revision History:
 
@@ -283,7 +284,7 @@ public:
         delete_table();
     }
 
-    void swap(core_hashtable & source) {
+    void swap(core_hashtable & source) noexcept {
         std::swap(m_table,       source.m_table);
         std::swap(m_capacity,    source.m_capacity);
         std::swap(m_size,        source.m_size);
@@ -405,8 +406,7 @@ public:
     }
 
     void insert(const data & e) {
-        data tmp(e);
-        insert(std::move(tmp));
+        insert(data(e));
     }
 
 #define INSERT_LOOP_CORE_BODY() {                                               \
@@ -462,8 +462,7 @@ public:
     }
 
     bool insert_if_not_there_core(const data & e, entry * & et) {
-        data temp(e);
-        return insert_if_not_there_core(std::move(temp), et);
+        return insert_if_not_there_core(data(e), et);
     }
 
     /**
@@ -472,7 +471,7 @@ public:
        that was already in the table.
      */
     data const & insert_if_not_there(data const & e) {
-        entry * et;
+        entry * et = nullptr;
         insert_if_not_there_core(e, et);
         return et->get_data();
     }
@@ -482,7 +481,7 @@ public:
        Return the entry that contains e.
     */
     entry * insert_if_not_there2(data const & e) {
-        entry * et;
+        entry * et = nullptr;
         insert_if_not_there_core(e, et);
         return et;
     }
@@ -639,6 +638,19 @@ public:
 
 #ifdef Z3DEBUG
     bool check_invariant() {
+        // The capacity must always be a power of two.
+        if (!is_power_of_two(m_capacity))
+            return false;
+
+        // The number of deleted plus the size must not exceed the capacity.
+        if (m_num_deleted + m_size > m_capacity)
+            return false;
+
+        // Checking that m_num_deleted is less than or equal to m_size.
+        if (m_num_deleted > m_size) {
+            return false;
+        }
+
         entry * curr = m_table;
         entry * end  = m_table + m_capacity;
         unsigned num_deleted = 0;
