@@ -16,10 +16,9 @@ Author:
 Revision History:
 
 --*/
-#ifndef PERMUTATION_H_
-#define PERMUTATION_H_
+#pragma once
 
-#include<iostream>
+#include<ostream>
 #include "util/vector.h"
 
 class permutation {
@@ -32,13 +31,15 @@ public:
     unsigned operator()(unsigned i) const { return m_p[i]; }
     unsigned inv(unsigned i_prime) const { return m_inv_p[i_prime]; }
 
-    void swap(unsigned i, unsigned j);
+    void swap(unsigned i, unsigned j) noexcept;
     void move_after(unsigned i, unsigned j);
     
     void display(std::ostream & out) const;
     bool check_invariant() const;
 };
 
+void swap(unsigned i, unsigned j) noexcept;
+
 inline std::ostream & operator<<(std::ostream & out, permutation const & p) {
     p.display(out);
     return out;
@@ -90,4 +91,3 @@ void apply_permutation(unsigned sz, T * data, unsigned const * p) {
     }
 }
 
-#endif
